if ( !GetSelectedMails ) var GetSelectedMails = {};

/**********
 * Add-on constants
 */

GetSelectedMails.PR_RDONLY		= 0x01;
GetSelectedMails.PR_WRONLY		= 0x02;
GetSelectedMails.PR_RDWR		= 0x04;
GetSelectedMails.PR_CREATE_FILE	= 0x08;
GetSelectedMails.PR_APPEND		= 0x10;
GetSelectedMails.PR_TRUNCATE	= 0x20;
GetSelectedMails.PR_SYNC		= 0x40;
GetSelectedMails.PR_EXCL		= 0x80;


/**
 * (class) Allows to use error console embedded in Mozilla apps
 */
GetSelectedMails.Console = function ()
{
	this.console = Components.classes["@mozilla.org/consoleservice;1"]
					.getService(Components.interfaces.nsIConsoleService);

	/**
	 * Adds 0-padding of given length to given number
	 *
	 * @param number (integer) number to format
	 * @param formatLength (integer) length of 0-padding to add
	 * @return (string) formatted number as string
	 */
	this.formatNumber = function (number, formatLength) {
		var paddingStr = "";
		var nmbrStr = "";
		var paddingLength = formatLength - number.toString().length;

		for ( var idx = 0 ; idx < paddingLength ; idx++ ) {
			paddingStr += "0";
		}
		nmbrStr = paddingStr + number.toString();

		return nmbrStr;
	}

	/**
	 * Formats date
	 *
	 * @param dt (Date object) date/time to format
	 * @return (string) formatted date as string
	 */
	this.formatDate = function (dt) {
		return dt.getUTCFullYear() + "-" + this.formatNumber(dt.getMonth() + 1, 2) + "-" + this.formatNumber(dt.getDate(), 2);
	}

	/**
	 * Formats time
	 *
	 * @param dt (Date object) date/time to format
	 * @return (string) formatted time as string
	 */
	this.formatTime = function (dt) {
		return this.formatNumber(dt.getHours(), 2) + ":" + this.formatNumber(dt.getMinutes(), 2) + ":" + this.formatNumber(dt.getSeconds(), 2) + "." + this.formatNumber(dt.getMilliseconds(), 3);
	}

	/**
	 *
	 */
	this.logInformation = function (source, message, forceFlag) {
		forceFlag = ( typeof(forceFlag) !== "undefined" ) ? forceFlag : false;

		if ( GetSelectedMails.debugMode || forceFlag ) {
			var dt = new Date();
			var dateTime = this.formatDate(dt) + " " + this.formatTime(dt);

			this.console.logStringMessage(dateTime + " [" + GetSelectedMails.addOnName + "." + source + "]: " + message);
			GetSelectedMails.writeMessageToFile("Information;" + dateTime + ";" + source + ";" + message);
		}
	}

	/**
	 *
	 */
	this.logError = function (source, message, noForceFlag) {
		noForceFlag = ( typeof(noForceFlag) !== "undefined" ) ? noForceFlag : true;

		var error = Components.classes["@mozilla.org/scripterror;1"]
						.createInstance(Components.interfaces.nsIScriptError);
		var dt = new Date();
		var dateTime = this.formatDate(dt) + " " + this.formatTime(dt);

		error.init(dateTime + " [" + GetSelectedMails.addOnName + "." + source + "]: " + message, "", null, null, null, Components.interfaces.nsIScriptError.errorFlag, null);

		this.console.logMessage(error);

		if ( noForceFlag ) {
			GetSelectedMails.writeMessageToFile("Error;" + dateTime + ";" + source + ";" + message);
		}
	}

	/**
	 *
	 */
	this.logWarning = function (source, message) {
		var warning = Components.classes["@mozilla.org/scripterror;1"]
						.createInstance(Components.interfaces.nsIScriptError);
		var dt = new Date();
		var dateTime = this.formatDate(dt) + " " + this.formatTime(dt);

		warning.init(dateTime + " [" + GetSelectedMails.addOnName + "." + source + "]: " + message, "", null, null, null, Components.interfaces.nsIScriptError.warningFlag, null);

		this.console.logMessage(warning);
		GetSelectedMails.writeMessageToFile("Warning;" + dateTime + ";" + source + ";" + message);
	}
};


/**
 * (class) Holds information about profile's settings
 *
 * @param id (string) profile id
 * @param name (string) profile name
 * @param button (boolean) true/false if custom button has to be displayed/hidden
 * @param title (string) button title
 * @param icon (string) button icon as color name
 * @param accounts (array of objects) list of selected accounts as (key,value): key=account, value=selected (false=unselected / true=selected)
 * @param hdrs (array of objects) headers fetching setting as (key,value): key=account, value=headers fetching value (0=keep default setting / 1=get entire messages / 2=fetch headers)
 * @param hdrsDflt (array of objects) headers fetching default setting as (key,value): key=account, value=headers fetching value as default (false=don't set as default / true=set as default)
 * @param srv (array of objects) server setting as (key,value): key=account, value=use mail server (false=don't use mail server / true=use mail server)
 * @param srvName (array of objects) server name as (key,value): key=account, value=mail server name
 * @param srvDflt (array of objects) server default setting as (key,value): key=account, value=mail server name as default (false=don't set as default / true=set as default)
 * @param port (array of objects) port setting as (key,value): key=account, value=use mail server port (false=don't use mail server port / true=use mail server port)
 * @param prtNmbr (array of objects) port number as (key,value): key=account, value=mail server port number
 * @param prtDflt (array of objects) port default setting as (key,value): key=account, value=mail server port as default (false=don't set as default / true=set as default)
 * @param sendBGet (boolean) true/false if unsent messages should be/should not be send before getting new messages
 */
GetSelectedMails.ProfileSettings = function (objid, name, button, title, icon, accounts, hdrs, hdrsDflt, srv, srvName, srvDflt, prt, prtNmbr, prtDflt, sendBGet)
{
	// attributes
	if ( typeof(objid) === 'object' ) { // if first parameter is an object (in this case this should be the one and only parameter)
		this.id = objid.id;
		this.name = objid.name;
		this.button = objid.button;
		this.title = objid.title;
		this.icon = objid.icon;
		this.accounts = objid.accounts;
		this.hdrs = objid.hdrs;
		this.hdrsDflt = objid.hdrsDflt;
		this.srv = objid.srv;
		this.srvName = objid.srvName;
		this.srvDflt = objid.srvDflt;
		this.prt = objid.prt;
		this.prtNmbr = objid.prtNmbr;
		this.prtDflt = objid.prtDflt;
		this.sendBGet = objid.sendBGet;
	}
	else { // if first parameter is not an object (in this case all other parameters should be defined)
		this.id = objid;
		this.name = name;
		this.button = button;
		this.title = title;
		this.icon = icon;
		this.accounts = accounts;
		this.hdrs = hdrs;
		this.hdrsDflt = hdrsDflt;
		this.srv = srv;
		this.srvName = srvName;
		this.srvDflt = srvDflt;
		this.prt = prt;
		this.prtNmbr = prtNmbr;
		this.prtDflt = prtDflt;
		this.sendBGet = sendBGet;
	}
};


/**
 * (class) Simplifies manipulation of lists between TB versions
 *
 * Supports nsISupportsArray (TB <= v19.*) and nsIArray (TB >= v20.0a1)
 *
 * @param obj (object) object to virtualize
 * @param uuid (nsIIDRef) type of object contained in list
 */
GetSelectedMails.VirtualArray = function (obj, uuid)
{
	// attributes
	this.realObj = obj;
	this.uuid = uuid;

	// methods
	this.count = function () {
		var cnt = -1;

		if ( this.realObj != null ) {
			if ( this.realObj.toString() === "[xpconnect wrapped nsISupportsArray]" ) {
				cnt = this.realObj.Count();
			}
			else if ( this.realObj.toString() === "[xpconnect wrapped nsIArray]" ) {
				cnt = this.realObj.length;
			}
			else {
				throw this.realObj.toString() + " is not a handled class"; // throws an exception
			}
		}

		return cnt;
	}

	this.queryAt = function (index) {
		var elt = null;

		if ( this.realObj != null ) {
			if ( this.realObj.toString() === "[xpconnect wrapped nsISupportsArray]" ) {
				elt = this.realObj.QueryElementAt(index, this.uuid);
			}
			else if ( this.realObj.toString() === "[xpconnect wrapped nsIArray]" ) {
				elt = this.realObj.queryElementAt(index, this.uuid);
			}
			else {
				throw this.realObj.toString() + " is not a handled class"; // throws an exception
			}
		}

		return elt;
	}
};


/**
 * (class) Simplifies manipulation of a local file between TB versions
 *
 * nsILocalFile was merged with the interface of nsIFile in Gecko 14
 * Much of the documentation has not been updated to reflect this change.
 * - TB <= v13.*
 *   (nsILocalFile)
 * - TB >= v14.0a1
 *   (nsIFile)
 *
 * @return (object) file object
 */
GetSelectedMails.VirtualLocalFile = function ()
{
	var fileObj = null;

	if ( GetSelectedMails.isTBVersionEqualOrHigherThan("14.0a1") ) { // if current TB version uses gecko engine v14.0
		fileObj = Components.classes["@mozilla.org/file/local;1"]
					.createInstance(Components.interfaces.nsIFile);
	}
	else {
		fileObj = Components.classes["@mozilla.org/file/local;1"]
					.createInstance(Components.interfaces.nsILocalFile);
	}

	return fileObj;
};


/**
 * (class) Holds information about detection rule
 *
 * @param objsIP (object/string) object / start ip
 * @param eIP (string) end ip
 * @param pId (string) profile id
 * @param on (boolean) true/false if detection rule is enabled/disabled
 */
GetSelectedMails.DetectionRule = function (objsIP, eIP, pId, on)
{
	if ( typeof(objsIP) === 'object' ) { // if first parameter is an object (in this case this should be the one and only parameter)
		this.sIP = objsIP.sIP;
		this.eIP = objsIP.eIP;
		this.pId = objsIP.pId;
		this.on = objsIP.on;
	}
	else { // if first parameter is not an object (in this case all other parameters should be defined)
		this.sIP = objsIP;
		this.eIP = eIP;
		this.pId = pId;
		this.on = on;
	}
};


/**********
 * Add-on variables
 */

GetSelectedMails.addOnName = "GetSelectedMails";
GetSelectedMails.addOnId = "GetSelectedMails@gmail.com";
GetSelectedMails.temporaryRoot = "GSM";
GetSelectedMails.exportSettingsMode = "PREFS"; // PREFS or INI

GetSelectedMails.localConsole = new GetSelectedMails.Console();
GetSelectedMails.stringBundle = null;

GetSelectedMails.profileAccountsSelections = "";
GetSelectedMails.profileNames = "";
GetSelectedMails.currentProfile = 0;
GetSelectedMails.currentDefaultProfileFlag = false;

GetSelectedMails.profilesSettings = "";
GetSelectedMails.defaultProfile = 0;
GetSelectedMails.mainButtonEnabled = true;
GetSelectedMails.autoDetectionAtStart = false;
GetSelectedMails.autoDetectionAtGetMails = false;
GetSelectedMails.autoDetectionAtIntervals = false;
GetSelectedMails.intervals = 5;
GetSelectedMails.detectionRules = "";
GetSelectedMails.delayEnabled = false;
GetSelectedMails.delay = 2;
GetSelectedMails.delayOnlyDeferredAccounts = true;
GetSelectedMails.nativeGetMailButtonOptionsEnabled = true;
GetSelectedMails.debugMode = false;
GetSelectedMails.debugModeToFile = false;
GetSelectedMails.debugFileName = "";

GetSelectedMails.maximumProfilesCount = 50;


/**********
 * Common methods
 */

/**
 * Gets Thunderbird version
 *
 * @param comparedVersion (string) version to compare current version to
 * @return (boolean) true/false if Thunderbird version is equal or higher/lower than given version
 */
GetSelectedMails.isTBVersionEqualOrHigherThan = function (comparedVersion)
{
	var version;
	var versionChecker;

	version = ( "@mozilla.org/xre/app-info;1" in Components.classes ) ?
		Components.classes["@mozilla.org/xre/app-info;1"]
			.getService(Components.interfaces.nsIXULAppInfo)
			.version
		:
		Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefBranch)
			.getCharPref("app.version");

	versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
						.getService(Components.interfaces.nsIVersionComparator);

	if ( versionChecker.compare(version, comparedVersion) >= 0 ) {
		return true;
	}

	return false;
};

/**
 * Displays given message in a message dialog
 *
 * @param message (string) message to be displayed
 */
GetSelectedMails.displayMessageDialog = function (message)
{
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
						.getService(Components.interfaces.nsIPromptService);

	promptService.confirmEx(window, GetSelectedMails.addOnName, message, promptService.BUTTON_TITLE_OK * promptService.BUTTON_POS_0, "", "", null, "", {value: false});
};

/**
 * Displays given message in a dialog
 *
 * @param message (string) message to be displayed
 */
GetSelectedMails.displayAlertDialog = function (message)
{
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
						.getService(Components.interfaces.nsIPromptService);

	promptService.alert(null, this.addOnName, message);
};

/**
 * Displays given message in a dialog and ask for confirmation
 *
 * @param message (string) message to be displayed
 * @param titleMsg (string) title to be displayed
 * @return (boolean) true/false if user has/has not confirmed
 */
GetSelectedMails.displayConfirmDialog = function (message, titleMsg)
{
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
						.getService(Components.interfaces.nsIPromptService);

	titleMsg = ( typeof(titleMsg) !== "undefined" ) ? titleMsg : this.addOnName;

	return promptService.confirm(null, titleMsg, message);
};

/**
 * Displays given message in a popup dialog
 *
 * @param message (string) message to be displayed
 */
GetSelectedMails.displayPopUpMessage = function (message)
{
	try {
		var alertService = Components.classes["@mozilla.org/alerts-service;1"]
							.getService(Components.interfaces.nsIAlertsService);

		alertService.showAlertNotification("chrome://GetSelectedMails/skin/icon.png", this.addOnName, message, false, "", null, "");
	}
	catch ( err ) {
		var windowWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
					.getService(Components.interfaces.nsIWindowWatcher)
					.openWindow(null, "chrome://global/content/alerts/alert.xul", "_blank", "chrome,titlebar=no,popup=yes", null);

		windowWatcher.arguments = ["chrome://GetSelectedMails/skin/icon.png", this.addOnName, message, false, ""];
	}};

/**
 * Loads and displays given page
 *
 * @param pageUrl (string) page URL to load and display
 */
GetSelectedMails.displayPage = function (pageUrl)
{
	var mail3PaneWindow = Components.classes["@mozilla.org/appshell/window-mediator;1"]
							.getService(Components.interfaces.nsIWindowMediator)
							.getMostRecentWindow("mail:3pane");

	if ( mail3PaneWindow != null ) {
		var tabmail = mail3PaneWindow.document.getElementById("tabmail");

		if ( tabmail != null ) { // if application supports tabs (TB >= v3.*)
			tabmail.openTab("contentTab", {contentPage: pageUrl});
		}
		else { // if application does not support tabs (TB < v3.*)
			window.opener.openDialog(pageUrl, "_blank", "chrome,resizable=yes,height=800,width=1000");
		}
	}
};

// for TB v3.* there is a bug in native functions JSON.stringify() and JSON.parse() (checked from Shredder v3.0.6 to Lanikai 3.1.21):
// when using an object and a replacer function, properties values of the object are not returned correctly:
// they stay unchanged, even if modified into replacer function

/**
 * Turns given object into a JSON string using JSON engine
 *
 * Aimed to replace native function JSON.stringify() for TB v3.*:
 * First the names are escaped (using escape()), then the whole object is stringified using native JSON function
 *
 * @param obj (object) object to stringify
 * @param replacer (function) function to apply
 * @param keysArray (array of strings) list of properties to apply function to
 * @return (string) JSON
 */
GetSelectedMails.JSONstringify = function (obj, replacer, keysArray)
{
	if ( typeof(obj) !== 'object' ) {
		throw "Error in function JSONstringify(): Argument is not a valid object but a " + typeof(obj);
	}

	if ( (replacer != null ) && (obj != null) ) { // if replacement function is defined and given object is not null
		if ( typeof(obj.length) === 'number' ) { // if given object is an array
			for ( var idx = 0 ; idx < obj.length ; idx++ ) { // for each element
				for ( var propty in obj[idx] ) { // for each property of current element
					if ( keysArray.indexOf(propty) != -1 ) { // if property is one of allowed keys
						obj[idx][propty] = replacer(propty, obj[idx][propty]); // applies replacement function
					}
				}
			}
		}
		else { // if given object is a single object
			for ( var propty in obj ) { // for each property of current element
				if ( keysArray.indexOf(propty) != -1 ) { // if property is one of allowed keys
					obj[propty] = replacer(propty, obj[propty]); // applies replacement function
				}
			}
		}
	}

	return JSON.stringify(obj);
};

/**
 * Gets a JSON object from given string considered as a JSON string
 *
 * Aimed to replace native function JSON.parse() for TB v3.*:
 * First the string is parsed using native JSON function, then names are unescaped (using unescape())
 *
 * @param str (string) string to parse
 * @param replacer (function) function to apply
 * @param keysArray (array of strings) list of properties to apply function to
 * @return (object) resulting object
 */
GetSelectedMails.JSONparse = function (str, replacer, keysArray)
{
	if ( typeof(str) !== 'string' ) {
		throw "Error in function JSONparse(): Argument '" + str + "' is not a valid string but a " + typeof(str);
	}
	else if ( str === "" ) {
		throw "Error in function JSONparse(): Argument is an empty string";
	}

	var obj = JSON.parse(str);

	if ( (replacer != null ) && (obj != null) ) { // if replacement function is defined and JSON-parsed object is not null
		if ( typeof(obj.length) === 'number' ) { // if JSON-parsed object is an array
			for ( var idx = 0 ; idx < obj.length ; idx++ ) { // for each element
				for ( var propty in obj[idx] ) { // for each property of current element
					if ( keysArray.indexOf(propty) != -1 ) { // if property is one of allowed keys
						obj[idx][propty] = replacer(propty, obj[idx][propty]); // applies replacement function
					}
				}
			}
		}
		else { // if JSON-parsed object is a single object
			for ( var propty in obj ) { // for each property of current element
				if ( keysArray.indexOf(propty) != -1 ) { // if property is one of allowed keys
					obj[propty] = replacer(propty, obj[propty]); // applies replacement function
				}
			}
		}
	}

	return obj;
};

/**
 * Applies unescape() function on string properties
 *
 * @param key (string) property key
 * @param value (string) property value
 * @return (string) modified property value
 */
GetSelectedMails.preProcessLoadProfileSettings = function (key, value)
{
	if ( typeof(value) === 'string' ) { // if property is a string
		value = unescape(value);
	}

	return value;
};

/**
 * Applies escape() function on string properties
 *
 * @param key (string) property key
 * @param value (string) property value
 * @return (string) modified property value
 */
GetSelectedMails.preProcessSaveProfileSettings = function (key, value)
{
	if ( typeof(value) === 'string' ) { // if property is a string
		value = escape(value);
	}

	return value;
};

/**
 * Writes given message to log file
 *
 * @param message (string) message to log
 */
GetSelectedMails.writeMessageToFile = function (message)
{
	if ( GetSelectedMails.debugMode && GetSelectedMails.debugModeToFile && (GetSelectedMails.debugFileName != "") ) {
		try {
			// check existence of debug log file
			var debugFile = GetSelectedMails.VirtualLocalFile();
			debugFile.initWithPath(GetSelectedMails.debugFileName);

			if ( !debugFile.exists() ) {
				debugFile.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0664);
			}

			// writing of message into debug log file
			var debugFileStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
									.createInstance(Components.interfaces.nsIFileOutputStream);
			debugFileStream.init(debugFile, GetSelectedMails.PR_WRONLY | GetSelectedMails.PR_APPEND, 0664, 0);

			var converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"]
							.createInstance(Components.interfaces.nsIConverterOutputStream);
			converter.init(debugFileStream, "UTF-8", 0, 0);
			converter.writeString(message + "\n");
			converter.close(); // closes debugFileStream too
		}
		catch ( err ) {
			GetSelectedMails.debugModeToFile = false;

			GetSelectedMails.localConsole.logError("writeMessageToFile", "Error while writing log message to debug file \"" + message + "\" (" + err + ")", false);
		}
	}
};
