if ( !GetSelectedMails ) var GetSelectedMails = {};


/**********
 * Add-on variables
 */

GetSelectedMails.prefs = null;

// attributes are defined in GetSelectedMail0.js

GetSelectedMails.profilesSettingsList = null;
GetSelectedMails.profilesListChanged = true;

GetSelectedMails.hiddenCustomButtons = null;

GetSelectedMails.timersArray = null;
GetSelectedMails.detectTimer = null;


/**********
 * Command methods
 */

GetSelectedMails.onButtonClicked = function ()
{
	this.currentProfile = this.defaultProfile;
	this.currentDefaultProfileFlag = true;

	this.launchGetMessages();
};

/**
 * Reads add-on settings
 */
GetSelectedMails.readSettings = function ()
{
	this.profilesSettings = this.prefs.getCharPref("profilesSettings");
	this.defaultProfile = this.prefs.getIntPref("defaultProfile");
	this.autoDetectionAtStart = this.prefs.getBoolPref("autoDetectionAtStart");
	this.autoDetectionAtGetMails = this.prefs.getBoolPref("autoDetectionAtGetMails");
	this.autoDetectionAtIntervals = this.prefs.getBoolPref("autoDetectionAtIntervals");
	this.intervals = this.prefs.getIntPref("intervals");
	this.detectionRules = this.prefs.getCharPref("detectionRules");
	this.delayEnabled = this.prefs.getBoolPref("delayEnabled");
	this.delay = this.prefs.getIntPref("delay");
	this.delayOnlyDeferredAccounts = this.prefs.getBoolPref("delayOnlyDeferredAccounts");
	this.mainButtonEnabled = this.prefs.getBoolPref("mainButtonEnabled");
	this.nativeGetMailButtonOptionsEnabled = this.prefs.getBoolPref("nativeGetMailButtonOptionsEnabled");
	this.debugMode = this.prefs.getBoolPref("debugMode");
	this.debugModeToFile = this.prefs.getBoolPref("debugModeToFile");
	this.debugFileName = unescape(this.prefs.getCharPref("debugFileName")); // decodes string
};

/**
 * Gets simple account id
 * 'accountX' -> 'aX'
 *
 * @param accountKey (string) account key
 * @return (string) simplified account key
 */
GetSelectedMails.getSimpleAccountId = function (accountKey)
{
	var accountKeyRgx = accountKey.match(new RegExp("^account([0-9]+)$"));
	var simpleAccountId = "";

	if ( accountKeyRgx != null ) {
		simpleAccountId = "a" + accountKeyRgx[1];
	}

	return simpleAccountId;
}

/**
 * Transfers old attributes to new attributes (their name was too long)
 *
 * @param obj (object) associativ array to handle
 * @return (string) new version of associativ array
 */
GetSelectedMails.cleanUpdateSubRoutine1 = function (obj)
{
	var newObj = {};
	var simpleAccountKey = "";

	for ( var accountKey in obj ) {
		simpleAccountKey = this.getSimpleAccountId(accountKey); // gets the 'aX' matching the 'accountX'

		if ( simpleAccountKey != "" ) {
			newObj[simpleAccountKey] = obj[accountKey];
		}
	}

	return newObj;
};

/**
 * Performs some cleaning when addon is updated
 */
GetSelectedMails.cleanUpdate = function ()
{
	var accountManager = Components.classes["@mozilla.org/messenger/account-manager;1"]
							.getService(Components.interfaces.nsIMsgAccountManager);

	var addonVersionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
								.getService(Components.interfaces.nsIVersionComparator);

	if ( this.prefs.prefHasUserValue("version") ) {
		// [v0 ; v0.5.0a1[
		if ( addonVersionChecker.compare(this.prefs.getCharPref("version"), "0.5.0a1") < 0 ) {
			// - converts profiles accounts selection (profilesAccountsSelection="profile1account1,profile1account2:profile2account1,profile2account3:...") into a profiles settings object
			// - adds a default name to each profile
			// - stores profiles settings as JSON string
			// - deletes preference 'profilesAccountsSelection'
			if ( this.prefs.prefHasUserValue("profilesAccountsSelection") ) {
				var profilesSettingsOld = this.prefs.getCharPref("profilesAccountsSelection");

				if ( profilesSettingsOld != "" ) { // if previous profiles settings string is not empty
					var accounts = new GetSelectedMails.VirtualArray(accountManager.accounts, Components.interfaces.nsIMsgAccount); // accounts list
					var account = null; // account object
					var accountServer = null; // related server
					var accountsKeys = new Array();
					var accountKey = "";

					var accountsSelectionArrayOld = profilesSettingsOld.split(/:/g);
					var profileName = "";
					var profileButtonTitle = "";
					var accountsSelectionObj = null;
					var accountsSelectionValue = null;

					var profilesSettingsList = new Array();

					// retrieval of accounts key list
					for ( var idx = 0 ; idx < accounts.count() ; idx++ ) { // for each account of accounts list
						account = accounts.queryAt(idx); // gets account object
						accountServer = account.incomingServer; // gets related server

						if ( (accountServer != null) && (accountServer.type != "none") ) { // if account is not "Local Folder"
							accountsKeys.push(account.key); // stores account key
						}
					}

					// building of profiles settings object
					for ( var idx = 0 ; idx < accountsSelectionArrayOld.length ; idx ++ ) {
						accountsSelectionObj = {};

						// building of accounts selection list
						for ( var jdx = 0 ; jdx < accountsKeys.length ; jdx++ ) {
							accountKey = accountsKeys[jdx];
							accountsSelectionValue = ( accountsSelectionArrayOld[idx].split(/,/g).indexOf(accountsKeys[jdx]) > -1 );
							accountsSelectionObj[accountKey] = accountsSelectionValue; // adds accounts selection
						}

						// add of a new profile settings object
						profileName = this.stringBundle.getString('profileDefaultPrefix') + " " + (idx + 1);
						profileButtonTitle = this.stringBundle.getString('customButtonDefaultTitle')
													+ " '" + profileName + "'";

						profilesSettingsList.push(
							new GetSelectedMails.ProfileSettings(
								"p" + idx,				// id
								profileName,			// name
								false, 					// custom button setting ('disabled')
								profileButtonTitle, 	// custom button title
								"", 					// custom button icon
								accountsSelectionObj,	// accounts selection
								{},						// headers fetching settings (all set to 'keep default setting')
								{},						// headers fetching default settings (all set to 'disabled')
								{}, 					// default server settings (all set to 'disabled')
								{},						// default server name settings (all set to <empty string>)
								{},						// default server default settings (all set to 'disabled')
								{},						// default port settings (all set to 'disabled')
								{},						// default port number settings (all set to 0)
								{},						// default port default settings (all set to 'disabled')
								false)					// send before get setting
							); // adds a new profile settings object
					}

					this.prefs.setCharPref("profilesSettings", this.JSONstringify(profilesSettingsList, this.preProcessSaveProfileSettings, new Array("name", "customButtonTitle"))); // stores JSON-string-converted profiles settings list
				}
				else { // if previous profiles settings string is empty
					this.prefs.setCharPref("profilesSettings", ""); // stores empty string as profiles settings list

					this.localConsole.logWarning("cleanUpdate", "Previous profiles settings were empty");
				}

				this.prefs.clearUserPref("profilesAccountsSelection");

				this.localConsole.logWarning("cleanUpdate", "Updating addon from v" + this.prefs.getCharPref("version") + " to v0.5.0: some cleaning has been performed");
			}
		}

		// [v0.5.0a1 ; v0.6.0a1[
		if ( (addonVersionChecker.compare(this.prefs.getCharPref("version"), "0.5.0a1") >= 0) &&
			 (addonVersionChecker.compare(this.prefs.getCharPref("version"), "0.6.0a1") < 0) )
		{
			// - converts profiles accounts selection (profileAccountsSelections="profile1account1,profile1account2;profile2account1,profile2account3;...") and names (profileNames="profile1name;profile2name;...") into a profiles settings object
			// - stores profiles settings as JSON string
			// - deletes preferences 'profileAccountsSelections' and 'profileNames'
			if ( this.prefs.prefHasUserValue("profileAccountsSelections") && this.prefs.prefHasUserValue("profileNames") ) {
				var profilesSettingsOld = this.prefs.getCharPref("profileAccountsSelections");

				if ( profilesSettingsOld != "" ) { // if previous profiles settings string is not empty
					var accounts = new GetSelectedMails.VirtualArray(accountManager.accounts, Components.interfaces.nsIMsgAccount); // accounts list
					var account = null; // account object
					var accountServer = null; // related server
					var accountsKeys = new Array();
					var accountKey = "";

					var accountsSelectionArrayOld = profilesSettingsOld.split(/;/g);
					var profileNamesArray = this.prefs.getCharPref("profileNames").split(/;/g);
					var profileName = "";
					var profileButtonTitle = "";
					var accountsSelectionObj = null;
					var accountsSelectionValue = null;

					var profilesSettingsList = new Array();

					// retrieval of accounts key list
					for ( var idx = 0 ; idx < accounts.count() ; idx++ ) { // for each account of accounts list
						account = accounts.queryAt(idx); // gets account object
						accountServer = account.incomingServer; // gets related server

						if ( (accountServer != null) && (accountServer.type != "none") ) { // if account is not "Local Folder"
							accountsKeys.push(account.key); // stores account key
						}
					}

					// building of profiles settings object
					for ( var idx = 0 ; idx < accountsSelectionArrayOld.length ; idx ++ ) {
						accountsSelectionObj = {};

						// building of accounts selection list
						for ( var jdx = 0 ; jdx < accountsKeys.length ; jdx++ ) {
							accountKey = accountsKeys[jdx];
							accountsSelectionValue = ( accountsSelectionArrayOld[idx].split(/,/g).indexOf(accountsKeys[jdx]) > -1 );
							accountsSelectionObj[accountKey] = accountsSelectionValue; // adds accounts selection
						}

						// add of a new profile settings object
						profileName = ( idx < profileNamesArray.length ) ? unescape(profileNamesArray[idx]) : ""; // ensures name exist
						profileButtonTitle = this.stringBundle.getString('customButtonDefaultTitle') + " '" + profileName + "'";

						profilesSettingsList.push(
							new GetSelectedMails.ProfileSettings(
								"p" + idx,				// id
								profileName,			// name
								false, 					// custom button setting ('disabled')
								profileButtonTitle, 	// custom button title
								"", 					// custom button icon
								accountsSelectionObj,	// accounts selection
								{},						// headers fetching settings (all set to 'keep default setting')
								{},						// headers fetching default settings (all set to 'disabled')
								{}, 					// default server settings (all set to 'disabled')
								{},						// default server name settings (all set to <empty string>)
								{},						// default server default settings (all set to 'disabled')
								{},						// default port settings (all set to 'disabled')
								{},						// default port number settings (all set to 0)
								{},						// default port default settings (all set to 'disabled')
								false)					// send before get setting
						); // adds a new profile settings object
					}

					this.prefs.setCharPref("profilesSettings", this.JSONstringify(profilesSettingsList, this.preProcessSaveProfileSettings, new Array("name", "customButtonTitle"))); // stores JSON-string-converted profiles settings list

					this.localConsole.logWarning("cleanUpdate", "Updating addon from v" + this.prefs.getCharPref("version") + " to v0.6.0: some cleaning has been performed");
				}
				else { // if previous profiles settings string is empty
					this.prefs.setCharPref("profilesSettings", ""); // stores empty string as profiles settings list

					this.localConsole.logWarning("cleanUpdate", "Previous profiles settings were empty");
				}
			}

			if ( this.prefs.prefHasUserValue("profileAccountsSelections") ) {
				this.prefs.clearUserPref("profileAccountsSelections");
			}
			if ( this.prefs.prefHasUserValue("profileNames") ) {
				this.prefs.clearUserPref("profileNames");
			}
		}

		// [v0.6.0a1 ; v0.9.0a1[
		if ( (addonVersionChecker.compare(this.prefs.getCharPref("version"), "0.6.0a1") >= 0) &&
			 (addonVersionChecker.compare(this.prefs.getCharPref("version"), "0.9.0a1") < 0) )
		{
			// - simplifies the way array objects are stored in JSON string using associativ arrays
			var profilesSettings = this.prefs.getCharPref("profilesSettings");
			var profilesSettingsObj = null;

			if ( profilesSettings != "" ) { // if profiles settings string is not empty
				var accountKey = "";
				var accountsSelectionObj = null;
				var headersFetchingObj = null;

				try {
					profilesSettingsObj = this.JSONparse(profilesSettings, this.preProcessLoadProfileSettings, new Array("name", "customButtonTitle")); // parses profiles settings as JSON string

					for ( var idx = 0 ; idx < profilesSettingsObj.length ; idx++ ) { // for each profile
						// custom button title (only for [v0.6.0a1 ; v0.7.0a1[)
						if ( addonVersionChecker.compare(this.prefs.getCharPref("version"), "0.7.0a1") < 0 ) {
							profilesSettingsObj[idx].customButtonTitle = this.stringBundle.getString('customButtonDefaultTitle') + " '" + profilesSettingsObj[idx].name + "'";
						}

						// accounts selection
						accountsSelectionObj = {};
						for ( var jdx = 0 ; jdx < profilesSettingsObj[idx].accountsSelection.length ; jdx++ ) {
							accountKey = profilesSettingsObj[idx].accountsSelection[jdx].key;
							accountsSelectionObj[accountKey] = profilesSettingsObj[idx].accountsSelection[jdx].value;
						}
						profilesSettingsObj[idx].accountsSelection = accountsSelectionObj;

						// headers fetching
						headersFetchingObj = {};
						for ( var jdx = 0 ; jdx < profilesSettingsObj[idx].headersFetching.length ; jdx++ ) {
							accountKey = profilesSettingsObj[idx].headersFetching[jdx].key;
							headersFetchingObj[accountKey] = profilesSettingsObj[idx].headersFetching[jdx].value;
						}
						profilesSettingsObj[idx].headersFetching = headersFetchingObj;

						// other settings
						profilesSettingsObj[idx].headersFetchingDefault = {};

						profilesSettingsObj[idx].server = {};
						profilesSettingsObj[idx].serverName = {};
						profilesSettingsObj[idx].serverDefault = {};

						profilesSettingsObj[idx].port = {};
						profilesSettingsObj[idx].portNumber = {};
						profilesSettingsObj[idx].portDefault = {};
					}

					profilesSettings = this.JSONstringify(profilesSettingsObj, this.preProcessSaveProfileSettings, new Array("name", "customButtonTitle")); // converts profiles settings list into JSON string
					this.prefs.setCharPref("profilesSettings", profilesSettings);

					this.localConsole.logWarning("cleanUpdate", "Updating addon from v" + this.prefs.getCharPref("version") + " to v0.9.0: some cleaning has been performed");
				}
				catch ( err ) {
					this.localConsole.logError("cleanUpdate", err);
				}

				if ( (profilesSettingsObj == null) || // if profiles settings string cannot be parsed or...
					 (typeof(profilesSettingsObj.length) !== 'number') ) // ... if profiles settings string cannot be parsed as an array
				{
					this.prefs.setCharPref("profilesSettings", ""); // reinitializes profiles settings string

					this.localConsole.logError("cleanUpdate", "Profiles settings is damaged: '" + profilesSettings + "'");
				}
			}
		}

		// [v0 ; v1.1.0a1[
		if ( addonVersionChecker.compare(this.prefs.getCharPref("version"), "1.1.0a1") < 0 ) {
			// - add the 'id' attribute to each profile
			// - transfers old attributes to new attributes (their name was too long)
			// - converts every key 'accountX' to 'aX' (too long)
			var profilesSettings = this.prefs.getCharPref("profilesSettings");
			var profilesSettingsObj = null;
			var profilesSettingsObjNew = new Array();

			if ( profilesSettings != "" ) { // if profiles settings string is not empty
				var simpleAccountKey = "";
				var accountsObj = null;
				var headersFetchingObj = null;

				try {
					profilesSettingsObj = this.JSONparse(profilesSettings, this.preProcessLoadProfileSettings, new Array("name", "customButtonTitle")); // parses profiles settings as JSON string

					for ( var idx = 0 ; idx < profilesSettingsObj.length ; idx++ ) { // for each profile
						profilesSettingsObjNew.push(
							new GetSelectedMails.ProfileSettings(
								"p" + idx,																		// id
								profilesSettingsObj[idx].name,													// name
								profilesSettingsObj[idx].customButton,											// custom button
								profilesSettingsObj[idx].customButtonTitle,										// custom button title
								profilesSettingsObj[idx].customButtonIcon,										// custom button icon
								this.cleanUpdateSubRoutine1(profilesSettingsObj[idx].accountsSelection),		// accounts lits
								this.cleanUpdateSubRoutine1(profilesSettingsObj[idx].headersFetching),			// headers fetching
								this.cleanUpdateSubRoutine1(profilesSettingsObj[idx].headersFetchingDefault),	// headers fetching default
								this.cleanUpdateSubRoutine1(profilesSettingsObj[idx].server),					// server
								this.cleanUpdateSubRoutine1(profilesSettingsObj[idx].serverName),				// server name
								this.cleanUpdateSubRoutine1(profilesSettingsObj[idx].serverDefault),			// server default
								this.cleanUpdateSubRoutine1(profilesSettingsObj[idx].port),						// port
								this.cleanUpdateSubRoutine1(profilesSettingsObj[idx].portNumber),				// port number
								this.cleanUpdateSubRoutine1(profilesSettingsObj[idx].portDefault),				// port default
								profilesSettingsObj[idx].sendBeforeGet)											// send before get
						); // adds a new profile settings object
					}

					profilesSettings = this.JSONstringify(profilesSettingsObjNew, this.preProcessSaveProfileSettings, new Array("name", "title")); // converts profiles settings list into JSON string
					this.prefs.setCharPref("profilesSettings", profilesSettings);

					this.localConsole.logWarning("cleanUpdate", "Updating addon from v" + this.prefs.getCharPref("version") + " to v1.1.0: some cleaning has been performed");
				}
				catch ( err ) {
					this.localConsole.logError("cleanUpdate", err);
				}

				if ( (profilesSettingsObj == null) || // if profiles settings string cannot be parsed or...
					 (typeof(profilesSettingsObj.length) !== 'number') ) // ... if profiles settings string cannot be parsed as an array
				{
					this.prefs.setCharPref("profilesSettings", ""); // reinitializes profiles settings string

					this.localConsole.logError("cleanUpdate", "Profiles settings is damaged: '" + profilesSettings + "'");
				}
			}
		}
	}
};

/**
 * Initializes startup
 */
GetSelectedMails.initializeOverlayStartup = function ()
{
	if ( this.isTBVersionEqualOrHigherThan("3.3a1pre") ) { // if current TB version uses gecko engine v2.0 (starting from TB v3.3a1pre)
		Components.utils.import("resource://gre/modules/AddonManager.jsm");
		AddonManager.getAddonByID(this.addOnId, function (addon) { // asks for addon id
			GetSelectedMails.overlayStartup(addon.version);
		});
	}
	else {
		var extensionManager = Components.classes["@mozilla.org/extensions/manager;1"]
								.getService(Components.interfaces.nsIExtensionManager);
		this.overlayStartup(extensionManager.getItemForID(this.addOnId).version);
	}

	// for TB version >= 3.3a1pre process will be pursued in overlayStartup() as soon as addon manager has loaded information (AddonManager.getAddonByID())
	// for TB version < 3.3a1pre process is pursued instantly
};

GetSelectedMails.checkMainButton = function (buttonIdName)
{
	var mainButtonEnabledValue = ( document.getElementById(buttonIdName) != null );
	this.prefs.setBoolPref("mainButtonEnabled", mainButtonEnabledValue);
};

/**
 * Displays/Hides given button
 *
 * @param buttonIdName (string) button id name as string
 * @param displayed (boolean) true/false to display/hide button
 * @param nextButtonIdName (string) button id name as string of next button
 */
GetSelectedMails.displayCustomButton = function (buttonIdName, displayed, nextButtonIdName)
{
	var toolbar = document.getElementById("mail-bar3"); // main toolbar for TB v3

	if ( toolbar == null ) { // if main toolbar is null (= this is not TB v3)
		toolbar = document.getElementById("mail-bar2"); // main toolbar for TB v2
	}

	if ( toolbar != null ) { // if toolbar is not null
		var customButton = document.getElementById(buttonIdName);

		if ( displayed ) { // if button has to be displayed
			var nextButton = document.getElementById(nextButtonIdName); // next button reference

			if ( (customButton == null) && (nextButton != null) ) { // if next button is not null and custom button is not already added to toolbar
				var res = toolbar.insertItem(buttonIdName, nextButton); // adds button

				if ( res != null ) {
					this.localConsole.logInformation("displayCustomButton", "Button '" + buttonIdName + "' has been displayed next to '" + nextButtonIdName + "'");
				}
				else {
					this.localConsole.logWarning("displayCustomButton", "Impossible to add button '" + buttonIdName + "': restart TB");
				}
			}
		}
		else { // if button has to be hidden
			if ( customButton != null ) { // if custom button is already added to toolbar
				toolbar.removeChild(customButton); // removes button

				this.localConsole.logInformation("displayCustomButton", "Button '" + buttonIdName + "' has been hidden");
			}
			else {
				this.localConsole.logWarning("displayCustomButton", "Button '" + buttonIdName + "' cannot be found in the toolbar");
			}
		}

		toolbar.setAttribute('currentset', toolbar.currentSet);
		document.persist(toolbar.id, "currentset");
	}
};

/**
 * Initializes add-on when starting Thunderbird composing session
 *
 * @param currentVersion (string) addon version
 */
GetSelectedMails.overlayStartup = function (currentVersion)
{
	// system info
	var runtimeInfo = Components.classes["@mozilla.org/xre/app-info;1"]  
					.getService(Components.interfaces.nsIXULRuntime); 
	var osVersion = runtimeInfo.OS;
	var x3264 = ( runtimeInfo.is64Bit ) ? "64bits" : "32bits";
	var procInfo = runtimeInfo.XPCOMABI;

	var tbVersion = ( "@mozilla.org/xre/app-info;1" in Components.classes ) ?
		Components.classes["@mozilla.org/xre/app-info;1"]
			.getService(Components.interfaces.nsIXULAppInfo)
			.version
		:
		Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefBranch)
			.getCharPref("app.version");

	this.localConsole.logInformation("initializeOverlayStartup", "OS: " + osVersion + " " + x3264 + " (" + procInfo + ")", true);
	this.localConsole.logInformation("initializeOverlayStartup", "Thunderbird: v" + tbVersion, true);
	this.localConsole.logInformation("initializeOverlayStartup", this.addOnName + ": v" + currentVersion, true);

	// startup
	this.stringBundle = document.getElementById("getSelectedMailsStringBundle");

	// initialization of preferences
	this.prefs = Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefService)
					.getBranch("extensions." + this.addOnName + ".");

	// nsIPrefBranch2 allows to observe for changes to preferences
	// During Gecko 13 development, nsIPrefBranch2 was deprecated, and its methods moved to nsIPrefBranch
	// Calling .QueryInterface(Components.interfaces.nsIPrefBranch2) is no longer required, although it still works
	if ( this.isTBVersionEqualOrHigherThan("13.0a1") ) { // if current TB version uses gecko engine v13.0
		this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch);
	}
	else {
		this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
	}

	this.prefs.addObserver("", this, false); // receives notifications if any preference changes

	// checkings of profiles settings
	var profilesSettings = "";
	var defaultProfile = this.prefs.getIntPref("defaultProfile");

	this.cleanUpdate(); // makes some cleaning when updating addon version

	profilesSettings = this.prefs.getCharPref("profilesSettings"); // gets profiles settings as JSON string

	// checking if at least one profile exists
	if ( profilesSettings != "" ) { // if profiles settings string is not empty
		try {
			this.profilesSettingsList = this.JSONparse(profilesSettings, this.preProcessLoadProfileSettings, new Array("name", "title")); // parses profiles settings as JSON string
		}
		catch ( err ) {
			this.localConsole.logError("overlayStartup", err);
		}

		if ( (this.profilesSettingsList == null) || // if profiles settings string cannot be parsed or...
			 (typeof(this.profilesSettingsList.length) !== 'number') ) // ... if profiles settings string cannot be parsed as an array
		{
			this.localConsole.logError("overlayStartup", "Profiles settings is damaged: '" + profilesSettings + "'");
			profilesSettings = ""; // reinitializes profiles settings string
		}
	}

	if ( profilesSettings == "" ) { // if no profile is defined or profiles settings parsing has met an error
		var accountManager = Components.classes["@mozilla.org/messenger/account-manager;1"]
								.getService(Components.interfaces.nsIMsgAccountManager);
		var accounts = new GetSelectedMails.VirtualArray(accountManager.accounts, Components.interfaces.nsIMsgAccount); // accounts list

		var accountsSelectionObj = {};
		var headersFetchingObj = {};
		var headersFetchingDefaultObj = {};

		var serverObj = {};
		var serverNameObj = {};
		var serverDefaultObj = {};

		var portObj = {};
		var portNumberObj = {};
		var portDefaultObj = {};

		var account = null; // account object
		var accountServer = null; // related server

		var accountKeyRgx = null;
		var accountKey = "";

		for ( var idx = 0 ; idx < accounts.count() ; idx++ ) { // for each account of accounts list
			account = accounts.queryAt(idx); // gets account object
			accountServer = account.incomingServer; // gets related server

			if ( (accountServer != null) && (accountServer.type != "none") ) { // if account is not "Local Folder"
				accountKey = this.getSimpleAccountId(account.key);

				if ( accountKey != "" ) {
					accountsSelectionObj[accountKey] = true; // adds accounts selection

					if ( (account.incomingServer.type != "imap") && (account.incomingServer.type != "nntp") ) { // if account is not a 'nntp'/'imap'-type account ('fetch headers only' is not a valid parameter for 'nntp' and 'imap' accounts)
						headersFetchingObj[accountKey] = 0; // adds headers fetching value
						headersFetchingDefaultObj[accountKey] = false; // adds headers fetching default value
					}

					serverObj[accountKey] = false; // adds server setting value
					serverNameObj[accountKey] = ""; // adds server name value
					serverDefaultObj[accountKey] = false; // adds server default value

					portObj[accountKey] = false; // adds port setting value
					portNumberObj[accountKey] = ""; // adds port value
					portDefaultObj[accountKey] = false; // adds port default value
				}
				else {
					this.localConsole.logWarning("overlayStartup", account.key + " is not a valid account key");
				}
			}
		}

		// storing of default profile settings
		var defaultProfileName = this.stringBundle.getString('profileDefaultPrefix') + " 1"; // creates a default profile name
		var defaultProfileButtonTitle = this.stringBundle.getString('customButtonDefaultTitle') + " '" + defaultProfileName + "'"; // creates default profile button title

		this.profilesSettingsList = new Array();
		this.profilesSettingsList.push(new GetSelectedMails.ProfileSettings( // adds a new profile settings object
			"p0",						// default id
			defaultProfileName,			// default profile name
			false,						// default custom button setting ('disabled')
			defaultProfileButtonTitle,	// default custom button title
			"",							// default custom button icon
			accountsSelectionObj, 		// default profile accounts selection (all accounts)
			headersFetchingObj,			// default profile headers fetching settings (all set to 'keep default setting')
			headersFetchingDefaultObj,	// default profile headers fetching default settings (all set to 'disabled')
			serverObj, 					// default server settings (all set to 'disabled')
			serverNameObj,				// default server name settings (all set to <empty string>)
			serverDefaultObj,			// default server default settings (all set to 'disabled')
			portObj,					// default port settings (all set to 'disabled')
			portNumberObj,				// default port number settings (all set to 0)
			portDefaultObj,				// default port default settings (all set to 'disabled')
			false));					// send before get setting

		this.localConsole.logWarning("overlayStartup", "Default profiles settings have been initialized");
	}

	// checking if default profile is inside limits
	if ( (defaultProfile < 0) || (defaultProfile > this.profilesSettingsList.length) ) {
		this.prefs.setIntPref("defaultProfile", 0);
	}

	// checking if every profile has a name
	for ( var idx = 0 ; idx < this.profilesSettingsList.length ; idx++ ) {
		if ( this.profilesSettingsList[idx].name == "" ) {
			this.profilesSettingsList[idx].name = this.stringBundle.getString('profileDefaultPrefix') + " " + (idx + 1);
		}
	}

	// update of profiles settings
	profilesSettings = this.JSONstringify(this.profilesSettingsList, this.preProcessSaveProfileSettings, new Array("name", "title")); // converts profiles settings list into JSON string
	this.prefs.setCharPref("profilesSettings", profilesSettings);

	// management of first run or when updating addon
	if ( !this.prefs.prefHasUserValue("version") || (currentVersion != this.prefs.getCharPref("version")) ) { // if version is undefined (= first installation) or different from current version (= addon has just been updated)
		// add of addon toolbar button
		if ( !this.prefs.prefHasUserValue("version") ) { // if version is undefined (= first installation)
			this.displayCustomButton("getSelectedMailsButton", true, "button-newmsg"); // adds custom button to toolbar

			this.prefs.setBoolPref("mainButtonEnabled", "true");
		}
		// if version is defined, custom button is not added (nb: if it was already displayed, the button will stay displayed)

		// setting or update of addon version
		this.prefs.setCharPref("version", currentVersion);

		// display of help page
		this.displayPopUpMessage(this.addOnName + " v" + currentVersion + " has been installed");
		//this.displayPage("chrome://" + this.addOnName + "/content/help/help.htm");
	}

	this.readSettings(); // reads settings

	this.localConsole.logInformation("overlayStartup", "Profiles settings are: '" + this.profilesSettings + "'");

	// gui initializations
	this.createGetMailButtonOptions();

	var nativeMailMailToolbar2 = document.getElementById("mail-bar2"); // reference to native mail toolbar palette is obtained through palette properties of toolbox
	var nativeMailMailToolbar3 = document.getElementById("mail-bar3"); // reference to native mail toolbar palette is obtained through palette properties of toolbox
	var nativeMailMailMenubar = document.getElementById("mail-toolbar-menubar2"); // reference to native mail toolbar palette is obtained through palette properties of toolbox
	var nativeMailToolbarPalette = document.getElementById("mail-toolbox").palette; // reference to native mail toolbar palette is obtained through palette properties of toolbox
	var nativeMailCommands = document.getElementById("mailCommands"); // native mail commands

	var customButtonSearch = null;
	var customToolBarButton = null;
	var customToolBarButtonMenuItem = null;
	var customToolBarButtonMenuPopUp = null;

	var openSettingsDialogCommand = null;

	this.hiddenCustomButtons = new Array();

	this.checkMainButton("getSelectedMailsButton");

	// search of custom toolbars
	var toolbarsArray = new Array(nativeMailMailToolbar3, nativeMailMailToolbar2, nativeMailMailMenubar, nativeMailToolbarPalette);
	toolbarsArray = this.searchCustomToolbars(toolbarsArray);

	// if buttons are created dynamically they cannot be persistent (this is reinitialized at TB startup)
	// so buttons are just declared statically (see overlay.xul) and are set dynamically using following loop
	for ( var idx = 0 ; idx < this.maximumProfilesCount ; idx++ ) { // for each possible profile
		// settings of button
		customButtonSearch = this.findButtonIn("getSelectedMailsProfileButton" + idx, toolbarsArray);
		customToolBarButton = customButtonSearch.buttonReference;

		if ( customToolBarButton != null ) { // if custom button object exists
			// settings of menubutton
			customToolBarButton.setAttribute('class', "toolbarbutton-1");
			customToolBarButton.setAttribute('type', "menu-button");
			customToolBarButton.setAttribute('command', "getSelectedMailsGetProfileCmd" + idx);
			customToolBarButton.setAttribute('allowevents', "true");

			// settings of menupopup
			customToolBarButtonMenuPopUp = document.createElement("menupopup"); // creates a new menupopup

			customToolBarButton.appendChild(customToolBarButtonMenuPopUp); // adds menupopup to custom button

			// settings of menuitem
			customToolBarButtonMenuItem = document.createElement("menuitem"); // creates a new menuitem
			customToolBarButtonMenuItem.setAttribute('label', this.stringBundle.getString('profileCustomButtonOptionsText')); // sets menuitem label
			customToolBarButtonMenuItem.tooltipText = this.stringBundle.getString('profileCustomButtonOptionsTooltipText'); // sets tooltip text
			customToolBarButtonMenuItem.setAttribute('command', "getSelectedMailsOpenSettingsDialogCmd" + idx); // sets related command

			customToolBarButtonMenuPopUp.appendChild(customToolBarButtonMenuItem); // appends menu item to menupopup
		}

		// creation of 'settings dialog' profile command
		openSettingsDialogCommand = document.createElement("command"); // creates a new command
		openSettingsDialogCommand.setAttribute('id', "getSelectedMailsOpenSettingsDialogCmd" + idx); // sets id
		openSettingsDialogCommand.addEventListener("command", function() { window.openDialog('chrome://GetSelectedMails/content/options.xul', '', 'chrome,titlebar,toolbar,centerscreen,modal,dialog=yes', this.id); }, false); // sets related action

		nativeMailCommands.appendChild(openSettingsDialogCommand); // appends command to native mail commands
	}

	this.updateProfilesList(true);

	this.detectProfileAuto();
	this.launchDetectProfileTimer(this.autoDetectionAtIntervals);

	this.localConsole.logInformation("overlayStartup", "Initialization has been completed", true);
};

/**
 * Looks for custom toolbars and stores them in an array
 *
 * @param toolbarsArray (array of object) array of toolbars (might be null/empty)
 * @return (object) given array of toolbars completed with found custom toolbars
 */
GetSelectedMails.searchCustomToolbars = function (toolbarsArray)
{
	var mailToolBox = document.getElementById("mail-toolbox");

	if ( toolbarsArray == null ) {
		toolbarsArray = new Array();
	}

	if ( mailToolBox != null ) { // if native mail toolbox exists
		var xulElement = mailToolBox.firstChild; // initializes xul element to first child

		// gets index of given 'get' profile
		// '^': start of string
		// '__customToolbar_': litteral string "__customToolbar_"
		// '(.+)': name of custom toolbar -> group #1
		// '$': end of string
		var customToolbarIdStr = new RegExp("^__customToolbar_(.+)$");

		var xulEltId = "";

		// search of menupopup of native 'Get Mail' button
		while ( xulElement != null ) { // while there is a next element
			xulEltId = xulElement.id;
			if ( (xulElement.nodeName === "toolbar") && xulEltId.match(customToolbarIdStr) ) { // if element is a toolbar
				toolbarsArray.push(document.getElementById(xulEltId));
			}

			xulElement = xulElement.nextSibling; // gets next element
		}
	}

	return toolbarsArray;
};

/**
 * Creates options of native 'Get Mail' button
 */
GetSelectedMails.createGetMailButtonOptions = function ()
{
	var nativeGetMailButton = document.getElementById("button-getmsg");
	var nativeGetMailButtonMenuPopUp = document.getElementById("button-getMsgPopup");
	var nativeGetMailButtonMenuPopUpGetAllSeparator = document.getElementById("button-getAllNewMsgSeparator");

	if ( nativeGetMailButton != null ) { // if native 'Get Mail' button exists
		var xulElement = null;

		if ( nativeGetMailButtonMenuPopUp == null ) {
			xulElement = nativeGetMailButton.firstChild; // initializes xul element to first child (useful for search of menupopup)

			// search of menupopup of native 'Get Mail' button
			while ( (xulElement != null) && (nativeGetMailButtonMenuPopUp == null) ) { // while there is a next element and menupopup has not been found
				if ( xulElement.nodeName == "menupopup" ) { // if element is a menupopup
					nativeGetMailButtonMenuPopUp = xulElement; // sets menupopup reference
				}
				else {
					xulElement = xulElement.nextSibling; // gets next element
				}
			}
		}

		if ( (nativeGetMailButtonMenuPopUp != null) && (nativeGetMailButtonMenuPopUpGetAllSeparator == null) ) {
			xulElement = nativeGetMailButtonMenuPopUp.firstChild; // initializes xul element to first child (useful for search of menuseparator)

			// search of menuseparator of native 'Get Mail' button context menu
			while ( (xulElement != null) && (nativeGetMailButtonMenuPopUpGetAllSeparator == null) ) { // while there is a next element and menuseparator has not been found
				if ( xulElement.nodeName == "menuseparator" ) { // if element is a menupopup
					nativeGetMailButtonMenuPopUpGetAllSeparator = xulElement; // sets menupopup reference
				}
				else {
					xulElement = xulElement.nextSibling; // gets next element
				}
			}
		}
	}

	// creation of items for native 'Get Mail' menupopup
	if ( (nativeGetMailButtonMenuPopUp != null) && (nativeGetMailButtonMenuPopUpGetAllSeparator != null) ) {
		// creation of 'Get selected mails' menuitem
		var getMailsButtonGetMenuItem = document.createElement("menuitem"); // creates a new menuitem
		getMailsButtonGetMenuItem.setAttribute('id', "getSelectedMailsMainMenuItem"); // sets id
		getMailsButtonGetMenuItem.setAttribute('label', this.stringBundle.getString('nativeButtonOptionsLabel')); // sets menuitem label
		getMailsButtonGetMenuItem.tooltipText = this.stringBundle.getString('nativeButtonOptionsTooltipText'); // sets tooltip text
		getMailsButtonGetMenuItem.setAttribute('class', "menuitem-iconic"); // activates iconic mode
		getMailsButtonGetMenuItem.setAttribute('image', "chrome://GetSelectedMails/skin/icon_sm.png"); // sets icon
		getMailsButtonGetMenuItem.setAttribute('command', "getSelectedMailsCmd"); // sets related command

		// creation of 'Get this profile' menuitem
		var getMailsButtonGetProfileMenu = document.createElement("menu"); // creates a new menu
		getMailsButtonGetProfileMenu.setAttribute('id', "getSelectedMailsGetProfileMenu"); // sets id
		getMailsButtonGetProfileMenu.setAttribute('label', this.stringBundle.getString('getProfileMenuText')); // sets menu label
		getMailsButtonGetProfileMenu.tooltipText = this.stringBundle.getString('getProfileMenuToolTipText'); // sets tooltip text

		var getMailsButtonGetProfileMenuPopUp = document.createElement("menupopup"); // creates a new menupopup
		getMailsButtonGetProfileMenuPopUp.setAttribute('id', "getSelectedMailsGetProfileMenuPopUp2"); // sets id

		getMailsButtonGetProfileMenu.appendChild(getMailsButtonGetProfileMenuPopUp);

		// creation of 'Default profile' menuitem
		var getMailsButtonDefaultProfileMenu = document.createElement("menu"); // creates a new menu
		getMailsButtonDefaultProfileMenu.setAttribute('id', "getSelectedMailsDefaultProfileMenu"); // sets id
		getMailsButtonDefaultProfileMenu.setAttribute('label', this.stringBundle.getString('defaultProfileMenuText')); // sets menu label
		getMailsButtonDefaultProfileMenu.tooltipText = this.stringBundle.getString('defaultProfileMenuToolTipText'); // sets tooltip text

		var getMailsButtonDefaultProfileMenuPopUp = document.createElement("menupopup"); // creates a new menupopup
		getMailsButtonDefaultProfileMenuPopUp.setAttribute('id', "getSelectedMailsDefaultProfileMenuPopUp2"); // sets id

		getMailsButtonDefaultProfileMenu.appendChild(getMailsButtonDefaultProfileMenuPopUp);

		// creation of 'Settings' menuitem
		var getMailsButtonSettingsMenuItem = document.createElement("menuitem"); // creates a new menuitem
		getMailsButtonSettingsMenuItem.setAttribute('id', "getSelectedMailsSettingsMenuItem"); // sets id
		getMailsButtonSettingsMenuItem.setAttribute('label', this.stringBundle.getString('nativeButtonOptionsSubMenuLabel')); // sets menuitem label
		getMailsButtonSettingsMenuItem.tooltipText = this.stringBundle.getString('nativeButtonOptionsSubMenuTooltipText'); // sets tooltip text
		getMailsButtonSettingsMenuItem.setAttribute('command', "getSelectedMailsOptionsCmd"); // sets related command

		// creation of menuseparator
		var getMailsMenuSeparator = document.createElement("menuseparator"); // creates a new menuseparator
		getMailsMenuSeparator.setAttribute('id', "getSelectedMailsMenuSeparator"); // sets id

		// insertion of items into native 'Get Mail' menupopup
		nativeGetMailButtonMenuPopUp.insertBefore(getMailsMenuSeparator, nativeGetMailButtonMenuPopUpGetAllSeparator); // appends menuseparator to native 'Get Mail' button context menu
		nativeGetMailButtonMenuPopUp.insertBefore(getMailsButtonGetMenuItem, nativeGetMailButtonMenuPopUpGetAllSeparator); // appends menuitem to native 'Get Mail' button context menu
		nativeGetMailButtonMenuPopUp.insertBefore(getMailsButtonGetProfileMenu, nativeGetMailButtonMenuPopUpGetAllSeparator); // appends menu to native 'Get Mail' button context menu
		nativeGetMailButtonMenuPopUp.insertBefore(getMailsButtonDefaultProfileMenu, nativeGetMailButtonMenuPopUpGetAllSeparator); // appends menu to native 'Get Mail' button context menu
		nativeGetMailButtonMenuPopUp.insertBefore(getMailsButtonSettingsMenuItem, nativeGetMailButtonMenuPopUpGetAllSeparator); // appends menuitem to native 'Get Mail' button context menu

		// native items of menupopup which allow to retrieve new messages for individual accounts are dynamically added by TB later
	}
	else {
		this.localConsole.logWarning("createGetMailButtonOptions", "Native 'Get Mail' menupopup or menuseparator has not been found (removed from toolbar?)");
	}

	this.enableGetMailButtonOptions(this.nativeGetMailButtonOptionsEnabled); // changes status of 'Get Mail' button options
};

/**
 * Finds button reference with given id in given containers
 *
 * @param seekedId (string) id of seeked button
 * @param containersArray (object[]) array of container objects (toolbar, palette, etc.) to search in
 * @return (object) search results with 2 properties: 1. buttonReference (button reference), 2. containerReference (container reference)
 */
GetSelectedMails.findButtonIn = function (seekedId, containersArray)
{
	var buttonRef = null;
	var containerRef = null;

	var xulElement = null;

	for ( var idx = 0 ; (idx < containersArray.length) && (buttonRef == null) && (containerRef == null) ; idx++ ) {
		if ( containersArray[idx] != null ) { // if container exists
			// search of button in container
			xulElement = containersArray[idx].firstChild;

			while ( (xulElement != null) && (buttonRef == null) ) { // while there is a next element and button has not been found
				if ( xulElement.id == seekedId ) { // if element is seeked button
					buttonRef = xulElement; // saves button reference
					containerRef = containersArray[idx]; // saves container reference
				}

				xulElement = xulElement.nextSibling; // gets next element
			}
		}
	}

	return {buttonReference: buttonRef, containerReference: containerRef};
};

/**
 * Updates profiles list
 *
 * @param startupFlag (flag) true/false if function is called at startup or not (currently unused)
 */
GetSelectedMails.updateProfilesList = function (startupFlag)
{
	if ( this.profilesListChanged ) { // if subjects list has been changed
		var nativeMailCommands = document.getElementById("mailCommands"); // native mail commands
		var customButtonGetProfileMenuPopUp = document.getElementById("getSelectedMailsGetProfileMenuPopUp"); // menupopup of custom toolbar button
		var customButtonDefaultProfileMenuPopUp = document.getElementById("getSelectedMailsDefaultProfileMenuPopUp"); // menupopup of custom toolbar button
		var getMailsButtonGetProfileMenuPopUp = document.getElementById("getSelectedMailsGetProfileMenuPopUp2"); // menupopup of native 'Get Mail' toolbar button
		var getMailsButtonDefaultProfileMenuPopUp = document.getElementById("getSelectedMailsDefaultProfileMenuPopUp2"); // menupopup of native 'Get Mail' toolbar button

		this.profilesSettingsList = this.JSONparse(this.profilesSettings, this.preProcessLoadProfileSettings, new Array("name", "title")); // parses profiles settings as JSON string

		if ( nativeMailCommands != null ) { // (JSCTSABT) if graphical objects exist
			for ( var idx = 0 ; idx < this.profilesSettingsList.length ; idx++ ) { // for each profile
				// creation of 'get' profile command
				getProfileCommand = document.createElement("command"); // creates a new command
				getProfileCommand.setAttribute('id', "getSelectedMailsGetProfileCmd" + idx); // sets id
				getProfileCommand.addEventListener("command", function() { GetSelectedMails.onGetProfile(this.id); }, false); // sets related action

				nativeMailCommands.appendChild(getProfileCommand); // appends command to native mail commands
			}
		}

		if ( (customButtonGetProfileMenuPopUp != null) || (customButtonDefaultProfileMenuPopUp != null) || (getMailsButtonGetProfileMenuPopUp != null) || (getMailsButtonDefaultProfileMenuPopUp != null) ) {
			// clearance of existing items
			if ( customButtonGetProfileMenuPopUp != null ) { // if custom toolbar button is displayed
				while ( customButtonGetProfileMenuPopUp.firstChild ) { // for each item of menupopup
					customButtonGetProfileMenuPopUp.removeChild(customButtonGetProfileMenuPopUp.firstChild); // removes item
				}
			}

			if ( customButtonDefaultProfileMenuPopUp != null ) { // if custom toolbar button is displayed
				while ( customButtonDefaultProfileMenuPopUp.firstChild ) { // for each item of menupopup
					customButtonDefaultProfileMenuPopUp.removeChild(customButtonDefaultProfileMenuPopUp.firstChild); // removes item
				}
			}

			if ( getMailsButtonGetProfileMenuPopUp != null ) { // if native toolbar button is displayed
				while ( getMailsButtonGetProfileMenuPopUp.firstChild ) { // for each item of menupopup
					getMailsButtonGetProfileMenuPopUp.removeChild(getMailsButtonGetProfileMenuPopUp.firstChild); // removes item
				}
			}

			if ( getMailsButtonDefaultProfileMenuPopUp != null ) { // if native toolbar button is displayed
				while ( getMailsButtonDefaultProfileMenuPopUp.firstChild ) { // for each item of menupopup
					getMailsButtonDefaultProfileMenuPopUp.removeChild(getMailsButtonDefaultProfileMenuPopUp.firstChild); // removes item
				}
			}

			// filling of profiles list
			var getProfileCommand = null;
			var defaultProfileCommand = null;
			var getProfileButtonMenuItem = null;
			var getProfileButtonMenuItem2 = null;
			var defaultProfileButtonMenuItem = null;
			var defaultProfileButtonMenuItem2 = null;

			for ( var idx = 0 ; idx < this.profilesSettingsList.length ; idx++ ) { // for each profile
				// creation of 'get' profile menuitem
				getProfileButtonMenuItem = document.createElement("menuitem"); // creates a new menuitem
				getProfileButtonMenuItem.setAttribute('id', "getSelectedMailsGetProfile" + idx + "MenuItem"); // sets id
				getProfileButtonMenuItem.setAttribute('value', this.profilesSettingsList[idx].id); // sets profile id as menuitem value
				getProfileButtonMenuItem.setAttribute('label', this.profilesSettingsList[idx].name); // sets profile name as menuitem label
				getProfileButtonMenuItem.tooltipText = this.stringBundle.getString('getProfileMenuItemTooltipText'); // sets tooltip text
				getProfileButtonMenuItem.setAttribute('command', "getSelectedMailsGetProfileCmd" + idx); // sets related command

				// filling of context menu of menupopup of native 'Get Mail' button
				if ( getMailsButtonGetProfileMenuPopUp != null ) { // if menupopup of native 'Get Mail' is displayed
					getProfileButtonMenuItem2 = getProfileButtonMenuItem.cloneNode(true); // creates a cloned menuitem
					getProfileButtonMenuItem2.setAttribute('id', "getSelectedMailsGetProfile" + idx + "MenuItem2"); // changes id

					getMailsButtonGetProfileMenuPopUp.appendChild(getProfileButtonMenuItem2); // appends new menuitem to menupopup of native 'Get Mail' button
				}

				// filling of context menu of toolbar button
				if ( customButtonGetProfileMenuPopUp != null ) { // if toolbar button is displayed
					customButtonGetProfileMenuPopUp.appendChild(getProfileButtonMenuItem); // appends new menuitem to button context menu
				}

				// creation of default profile command
				defaultProfileCommand = document.createElement("command"); // creates a new command
				defaultProfileCommand.setAttribute('id', "getSelectedMailsDefaultProfileCmd" + idx); // sets id
				defaultProfileCommand.addEventListener("command", function() { GetSelectedMails.onDefaultProfileSelected(this.id); }, false); // sets related action

				nativeMailCommands.appendChild(defaultProfileCommand); // appends command to native mail commands

				// creation of default profile menuitem
				defaultProfileButtonMenuItem = document.createElement("menuitem"); // creates a new menuitem
				defaultProfileButtonMenuItem.setAttribute('id', "getSelectedMailsDefaultProfile" + idx + "MenuItem"); // sets id
				defaultProfileButtonMenuItem.setAttribute('type', "radio"); // sets menuitem type
				defaultProfileButtonMenuItem.setAttribute('name', "defaultProfile"); // sets name of radiogroup
				defaultProfileButtonMenuItem.setAttribute('value', this.profilesSettingsList[idx].id); // sets profile id as menuitem value
				defaultProfileButtonMenuItem.setAttribute('label', this.profilesSettingsList[idx].name); // sets profile name as menuitem label
				defaultProfileButtonMenuItem.tooltipText = this.stringBundle.getString('defaultProfileMenuItemTooltipText'); // sets tooltip text
				defaultProfileButtonMenuItem.setAttribute('command', "getSelectedMailsDefaultProfileCmd" + idx); // sets related command

				// filling of context menu of menupopup of native 'Get Mail' button
				// must be done before 'filling of context menu of toolbar button' because the menuitem should be cloned before being added to any toolbar
				if ( getMailsButtonDefaultProfileMenuPopUp != null ) { // if menupopup of native 'Get Mail' is displayed
					defaultProfileButtonMenuItem2 = defaultProfileButtonMenuItem.cloneNode(true); // creates a cloned menuitem
					defaultProfileButtonMenuItem2.setAttribute('id', "getSelectedMailsDefaultProfile" + idx + "MenuItem2"); // changes id

					getMailsButtonDefaultProfileMenuPopUp.appendChild(defaultProfileButtonMenuItem2); // appends new menuitem to menupopup of native 'Get Mail' button
				}

				// filling of context menu of toolbar button
				if ( customButtonDefaultProfileMenuPopUp != null ) { // if toolbar button is displayed
					customButtonDefaultProfileMenuPopUp.appendChild(defaultProfileButtonMenuItem); // appends new menuitem to button context menu
				}
			}
		}

		// management of custom icon
		var customToolBarButton = null;
		var customToolBarButtonContainer = null;
		var hiddenCustomButtonsTemp = null

		var nativeMailMailToolbar2 = document.getElementById("mail-bar2");
		var nativeMailMailToolbar3 = document.getElementById("mail-bar3");
		var nativeMailMailMenubar = document.getElementById("mail-toolbar-menubar2");
		var nativeMailToolbarPalette = document.getElementById("mail-toolbox").palette; // reference to native mail toolbar palette is obtained through palette properties of toolbox

		var customButtonSearch = null;
		var removedButton = null;

		if ( nativeMailToolbarPalette != null ) { // (JSCTSABT) if native toolbar palette has been found
			// search of custom toolbars
			var toolbarsArray = new Array(nativeMailMailToolbar3, nativeMailMailToolbar2, nativeMailMailMenubar, nativeMailToolbarPalette);
			toolbarsArray = this.searchCustomToolbars(toolbarsArray);

			for ( var idx = 0 ; idx < this.maximumProfilesCount ; idx++ ) { // for each possible profile
				customToolBarButton = null;
				customToolBarButtonContainer = null;

				// search of custom button in hidden buttons and (re)building of hidden buttons array
				// if custom button is found in hidden buttons, it is added to toolbar palette and removed from hidden buttons
				// it is later removed from toolbar/menubar/palette/etc. in case it is disabled or related profile has been deleted
				hiddenCustomButtonsTemp = new Array();

				for ( var jdx = 0 ; jdx < this.hiddenCustomButtons.length ; jdx++ ) { // for each possible hidden button
					if ( (this.profilesSettingsList[idx] != null) && (this.hiddenCustomButtons[jdx].id == ("getSelectedMailsProfileButton" + idx) && (customToolBarButton == null)) ) { // if profile exists and custom button has been found in hidden buttons
						customButtonSearch = {buttonReference: this.hiddenCustomButtons[jdx], containerReference: nativeMailToolbarPalette};
						customToolBarButton = customButtonSearch.buttonReference; // gets custom button
						customToolBarButtonContainer = customButtonSearch.containerReference; // gets custom button container
						nativeMailToolbarPalette.appendChild(customToolBarButton); // adds custom button to native mail toolbar palette
					}
					else {
						hiddenCustomButtonsTemp.push(this.hiddenCustomButtons[jdx]);
					}
				}
				this.hiddenCustomButtons = hiddenCustomButtonsTemp; // crushes hidden buttons array so that a potential enabled custom button for an existing profile is removed

				if ( customToolBarButton == null ) { // if current custom button has not been found in hidden buttons (= current custom button is not hidden)
					customButtonSearch = this.findButtonIn("getSelectedMailsProfileButton" + idx, toolbarsArray); // finds button in given toolbars, menubars, or palettes
					customToolBarButton = customButtonSearch.buttonReference; // gets custom button
					customToolBarButtonContainer = customButtonSearch.containerReference; // gets custom button container
				}

				// at this step any custom button of any possible profile is inside a toolbar/menubar/palette/etc.
				if ( customToolBarButton != null ) { // (JSCTSABT) if custom button object exists
					if ( (this.profilesSettingsList[idx] != null) && this.profilesSettingsList[idx].button ) { // if profile exists and custom button is enabled
						customToolBarButton.hidden = false;

						// customization of button
						customToolBarButton.setAttribute('label', this.profilesSettingsList[idx].title);
						customToolBarButton.tooltipText = this.stringBundle.getString('profileCustomButtonToolTipTextSta')
															+ " '" + this.profilesSettingsList[idx].name + "' "
															+ this.stringBundle.getString('profileCustomButtonToolTipTextEnd');
						customToolBarButton.style.setProperty("list-style-image", "url('chrome://GetSelectedMails/skin/GetSelectedMails_" + this.profilesSettingsList[idx].icon + ".png')", null);
					}
					else { // if profile does not exist or custom button is disabled
						removedButton = customToolBarButtonContainer.removeChild(customToolBarButton); // removes button from container
						this.hiddenCustomButtons.push(removedButton); // saves button object in hidden buttons

						customToolBarButtonContainer.setAttribute('currentset', customToolBarButtonContainer.currentSet);
						document.persist(customToolBarButtonContainer.id, "currentset");
					}
				}
			}
		}

		this.profilesListChanged = false; // marks subjects list as unchanged

		this.localConsole.logInformation("updateProfilesList", "Profiles list has been updated");
	}
};

/**
 * Selects default profile menuitems regarding default profile value
 *
 * @param defaultProfileIdx (int) default profile index
 */
GetSelectedMails.selectDefaultProfile = function (defaultProfileIdx)
{
	var defaultProfileMenuItem = document.getElementById("getSelectedMailsDefaultProfile" + defaultProfileIdx + "MenuItem");
	var defaultProfileMenuItem2 = document.getElementById("getSelectedMailsDefaultProfile" + defaultProfileIdx + "MenuItem2");

	if ( (defaultProfileMenuItem != null) || (defaultProfileMenuItem2 != null) ) { // if any of menuitem related to default profile selected exists

		if ( defaultProfileMenuItem != null ) { // if menuitem of toolbar button exists (= custom toolbar button is shown)
			defaultProfileMenuItem.setAttribute('checked', "true"); // checks menuitem of toolbar button
		}

		if ( defaultProfileMenuItem2 != null ) { // if menuitem of native 'Get Mail' button exists (= options of native button have been enabled)
			defaultProfileMenuItem2.setAttribute('checked', "true"); // checks menuitem of native 'Get Mail' button
		}

		this.defaultProfile = this.prefs.getIntPref("defaultProfile");

		this.localConsole.logInformation("selectDefaultProfile", "Profile #" + defaultProfileIdx + " / " + this.profilesSettingsList[defaultProfileIdx].id + " / '" + this.profilesSettingsList[defaultProfileIdx].name + "' has been set as default profile");
	}
	else { // no menuitem exists
		this.localConsole.logWarning("selectDefaultProfile", "Profile #" + defaultProfileIdx + " cannot be set as default: no available menuitem (some buttons removed?)");
	}
};

/**
 * Gets messages when a 'get profile' menuitem is selected
 *
 * @param getProfileRef (string) reference to selected 'get' profile
 */
GetSelectedMails.onGetProfile = function (getProfileRef)
{
	// gets index of given 'get' profile
	// '^': start of string
	// '[^0-9]+': anything but a figure 1 or several times
	// '([0-9]+)': 1 number (index) -> group #1
	// '$': end of string
	var getProfileIdx = getProfileRef.match(new RegExp("^[^0-9]+([0-9]+)$"));

	if ( getProfileIdx != null ) {
		this.currentProfile = getProfileIdx[1];
		this.currentDefaultProfileFlag = false;

		this.launchGetMessages(); // gets new messages
	}
};

/**
 * Updates default profile value when a default profile menuitem is selected
 *
 * @param defaultProfileRef (string) reference to selected default profile
 */
GetSelectedMails.onDefaultProfileSelected = function (defaultProfileRef)
{
	// gets index of given default profile
	// '^': start of string
	// '[^0-9]+': anything but a figure 1 or several times
	// '([0-9]+)': 1 number (index) -> group #1
	// '$': end of string
	var defaultProfileIdx = defaultProfileRef.match(new RegExp("^[^0-9]+([0-9]+)$"));

	if ( defaultProfileIdx != null ) {
		this.defaultProfile = defaultProfileIdx[1];
		this.prefs.setIntPref("defaultProfile", this.defaultProfile); // changes default profile
	}
};

/**
 * Enables or disables options for native 'Get Mail' button
 *
 * @param state (boolean) true/false if options for 'Get Mail' button must be enabled/disabled
 */
GetSelectedMails.enableGetMailButtonOptions = function (state)
{
	var getMailsMenuSeparator = document.getElementById("getSelectedMailsMenuSeparator");
	var getMailsButtonGetMenuItem = document.getElementById("getSelectedMailsMainMenuItem");
	var getMailsButtonGetProfileMenu = document.getElementById("getSelectedMailsGetProfileMenu");
	var getMailsButtonDefaultProfileMenu = document.getElementById("getSelectedMailsDefaultProfileMenu");
	var getMailsButtonSettingsMenuItem = document.getElementById("getSelectedMailsSettingsMenuItem");

	if ( (getMailsMenuSeparator != null) && (getMailsButtonGetMenuItem != null) && (getMailsButtonGetProfileMenu != null) && (getMailsButtonDefaultProfileMenu != null) && (getMailsButtonSettingsMenuItem != null) ) {
		getMailsMenuSeparator.hidden = !state;
		getMailsButtonGetMenuItem.hidden = !state;
		getMailsButtonGetProfileMenu.hidden = !state;
		getMailsButtonDefaultProfileMenu.hidden = !state;
		getMailsButtonSettingsMenuItem.hidden = !state;

		this.nativeGetMailButtonOptionsEnabled = state;
	}
}

/**
 * Launches 'get new messages' for selected accounts
 */
GetSelectedMails.launchGetMessages = function ()
{
	if ( this.autoDetectionAtGetMails ) {
		this.launchProfileDetection(true);
	}
	else {
		this.localConsole.logInformation("launchGetMessages", "Profile auto detection while getting messages is disabled");

		this.finalizeGetMessages();
	}
};

/**
 * Finalizes 'get new messages' for selected accounts
 */
GetSelectedMails.finalizeGetMessages = function ()
{
	var profileIdx = this.currentProfile;

	this.currentDefaultProfileFlag = false; // reinits the flag (useful for future 'get new messages' launchs

	if ( (profileIdx >= 0) && (profileIdx < this.profilesSettingsList.length) ) { // if profile index actually exists
		var iOService = Components.classes["@mozilla.org/network/io-service;1"]
						.getService(Components.interfaces.nsIIOService);
		var isOnLine = !iOService.offline;

		if ( !isOnLine ) { // if TB is offline
			if ( this.displayConfirmDialog(document.getElementById("bundle_offlinePrompts").getString('getMessagesOfflineLabel1'), document.getElementById("bundle_offlinePrompts").getString('getMessagesOfflineWindowTitle1')) ) {
				var offlineManager = Components.classes["@mozilla.org/messenger/offline-manager;1"]
										.getService(Components.interfaces.nsIMsgOfflineManager);

				offlineManager.goOnline(false, false, msgWindow); // gets back online

				isOnLine = true;
			}
		}

		if ( isOnLine ) { // if TB is online
			var tmpProfileSettings = new GetSelectedMails.ProfileSettings(this.profilesSettingsList[profileIdx]); // builds a copy of current profile settings object
			var accountsSelectionValue = null;

			this.localConsole.logInformation("finalizeGetMessages", "Selected accounts for current profile (#" + profileIdx + "): '" + this.JSONstringify(tmpProfileSettings.accounts) + "'");

			var accountManager = Components.classes["@mozilla.org/messenger/account-manager;1"]
									.getService(Components.interfaces.nsIMsgAccountManager);
			var accounts = new GetSelectedMails.VirtualArray(accountManager.accounts, Components.interfaces.nsIMsgAccount); // accounts list
			var account = null; // account object
			var accountServer = null; // related server

			var accountKeyRgx = null;
			var accountKey = "";

			var serverPrefs = null;
			var deferringAccount = "";

			var tmpAccountsSelectionValue = null;

			var firstEnabledAccount = false;

			var timerDelay = 0;
			var firstTime = true;

			var tmpDelay = 0; // delay counter for case 'delay enabled' and 'not only deferred accounts'
			var tmpDelay2 = 0; // delay counter for case 'delay enabled' and 'only deferred accounts'

			this.timersArray = new Array();

			// gets new messages for selected accounts of current profile
			for ( var idx = 0 ; idx < accounts.count() ; idx++ ) { // for each account of accounts list
				account = accounts.queryAt(idx); // gets account object
				accountServer = account.incomingServer; // gets related server

				if ( (accountServer != null) && (accountServer.type != "none") ) { // if account is not "Local Folder"
					accountKey = this.getSimpleAccountId(account.key);

					if ( accountKey != "" ) {
						accountsSelectionValue = tmpProfileSettings.accounts[accountKey];

						if ( (accountsSelectionValue != null) && accountsSelectionValue ) { // if account can be found in accounts selection string
							// send of unsent messages
							if ( tmpProfileSettings.sendBGet && !firstEnabledAccount ) {
								this.sendUnsentMessages(account);

								firstEnabledAccount = true;
							}

							// getting messages
							tmpDelay = 0;
							tmpDelay2 = 0;

							if ( this.delayOnlyDeferredAccounts ) { // if only deferred accounts should be delayed
								serverPrefs = Components.classes["@mozilla.org/preferences-service;1"]
												.getService(Components.interfaces.nsIPrefService)
												.getBranch("mail.server." + account.incomingServer.key + ".");
								serverPrefs.QueryInterface(Components.interfaces.nsIPrefBranch);

								if ( serverPrefs.prefHasUserValue("deferred_to_account") && (serverPrefs.getCharPref("deferred_to_account") != "") ) { // if current account is deferred
									deferringAccount = serverPrefs.getCharPref("deferred_to_account");
									tmpAccountsSelectionValue = tmpProfileSettings.accounts[deferringAccount];

									if ( (tmpAccountsSelectionValue != null) && tmpAccountsSelectionValue ) { // if account is deferred to another account which will also be retrieved
										tmpDelay2 = this.delay * 1000; // adds given delay
									}
								}
							}
							else if ( !firstTime ) { // if all accounts should be delayed and this is not the first time
								tmpDelay = this.delay * 1000; // adds given delay
							}

							if ( !this.delayEnabled || (!this.delayOnlyDeferredAccounts && (tmpDelay == 0)) || (this.delayOnlyDeferredAccounts && (tmpDelay2 == 0)) ) { // if delay has been disabled or current delay is 0
								this.getMessages(tmpProfileSettings, account); // launches 'get messages' right now

								firstTime = false;
							}
							else { // if a delay between retrievals has been set
								// process is pursued in another thread using a timer with given delay
								timerDelay += tmpDelay + tmpDelay2;

								this.timersArray[idx] = Components.classes["@mozilla.org/timer;1"]
														.createInstance(Components.interfaces.nsITimer);

								this.timersArray[idx].initWithCallback(new GetSelectedMails.TimerGetMessages(this.timersArray[idx], tmpProfileSettings, account), timerDelay, Components.interfaces.nsITimer.TYPE_ONE_SHOT); // programs 'get messages'

								this.localConsole.logInformation("finalizeGetMessages", "Getting of new messages for account '" + account.key + "' has been programed in " + timerDelay + " milliseconds");

								// here ends the function
								// process will be pursued in the "timer expired" handler (GetSelectedMails.TimerGetMessages.notify()) once timer will be expired
							}
						}
					}
					else {
						this.localConsole.logWarning("finalizeGetMessages", account.key + " is not a valid account key");
					}
				}
			}
		}
		else {
			this.localConsole.logWarning("finalizeGetMessages", "TB is offline: messages cannot be retrieved");
		}
	}
	else {
		this.localConsole.logError("finalizeGetMessages", "Profile #" + profileIdx + " does not exist");
	}
};

/**
 * Sends unsent messages
 */
GetSelectedMails.sendUnsentMessages = function (account)
{
	var msgSendlater = Components.classes["@mozilla.org/messengercompose/sendlater;1"]
						.getService(Components.interfaces.nsIMsgSendLater);

	var msgFolder = msgSendlater.getUnsentMessagesFolder(account.defaultIdentity);

	if ( msgFolder != null ) {
		if ( msgFolder.getTotalMessages(false) ) {
			msgSendlater.sendUnsentMessages(account.defaultIdentity);

			this.localConsole.logInformation("sendUnsentMessages", "Sending of messages from outbox has been launched...");
		}
	}
};

/**
 * Gets new messages for given account of given profile
 *
 * @param profileSettings (ProfileSettings object) settings of current profile as a ProfileSettings object
 * @param account (nsIMsgAccount) account object to retrieve new messages for
 */
GetSelectedMails.getMessages = function (profileSettings, account)
{
	var accountKey = "";
	var accountServer = null;

	var headersFetchingValue = null;
	var serverValue = null;
	var serverNameValue = "";
	var portValue = null;
	var portNumberValue = 0;

	var serverPrefs = null;
	var accountServerHeadersOnly = false;
	var tempAccountServerHeadersOnly = false;
	var accountServerRealHostName = "";
	var accountServerHostName = "";
	var accountServerPortExist = false;
	var accountServerPort = 0;

	// initializations
	accountKey = this.getSimpleAccountId(account.key);

	accountServer = account.incomingServer; // gets account server

	serverPrefs = Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefService)
					.getBranch("mail.server." + accountServer.key + ".");
	serverPrefs.QueryInterface(Components.interfaces.nsIPrefBranch);

	accountServerHeadersOnly = false;
	tempAccountServerHeadersOnly = false;

	accountServerRealHostName = "";
	accountServerHostName = "";

	// headers fetching handling
	headersFetchingValue = profileSettings.hdrs[accountKey];

	if ( (headersFetchingValue != null) && (headersFetchingValue != 0) ) { // if headers fetching value is not set to 'keep default value'
		accountServerHeadersOnly = serverPrefs.prefHasUserValue("headers_only") ? serverPrefs.getBoolPref("headers_only") : false; // backups default 'fetch headers only' value

		tempAccountServerHeadersOnly = ( headersFetchingValue == 2 );
		serverPrefs.setBoolPref("headers_only", tempAccountServerHeadersOnly); // forces 'fetch headers only' value to headers fetching value of current profile

		this.localConsole.logInformation("getMessages", "Headers fetching value (preference='headers_only') has been forced to '" + headersFetchingValue + "' for account '" + account.key + "'");
	}

	// server handling
	serverValue = profileSettings.srv[accountKey];
	serverNameValue = profileSettings.srvName[accountKey];

	if ( (serverValue != null) && serverValue && (serverNameValue != null) && (serverNameValue != "") ) { // if server value is enabled and server name is not empty
		// pop3 accounts use preference 'realhostname'
		// all other account types use preference 'hostname'
		if ( serverPrefs.prefHasUserValue("realhostname") ) {
			accountServerRealHostName = serverPrefs.getCharPref("realhostname"); // backups default 'server name' value

			serverPrefs.setCharPref("realhostname", serverNameValue); // forces 'realhostname' value to server name value of current profile

			this.localConsole.logInformation("getMessages", "Server value (preference='realhostname') has been forced to '" + serverNameValue + "' for account '" + account.key + "'");
		}
		else if ( serverPrefs.prefHasUserValue("hostname") ) {
			accountServerHostName = serverPrefs.getCharPref("hostname"); // backups default 'server name' value

			serverPrefs.setCharPref("hostname", serverNameValue); // forces 'hostname' value to server name value of current profile

			this.localConsole.logInformation("getMessages", "Server value (preference='hostname') has been forced to '" + serverNameValue + "' for account '" + account.key + "'");
		}
	}

	// port handling
	portValue = profileSettings.prt[accountKey];
	portNumberValue = profileSettings.prtNmbr[accountKey];

	if ( (portValue != null) && portValue && (portNumberValue != null) && (portNumberValue != 0) ) { // if port value is enabled and port is not 0
		accountServerPortExist = serverPrefs.prefHasUserValue("port");
		accountServerPort = ( accountServerPortExist ) ? serverPrefs.getIntPref("port") : 0; // backups default 'port' value

		serverPrefs.setIntPref("port", portNumberValue); // forces 'port' value to port number value of current profile

		this.localConsole.logInformation("getMessages", "Port number value (preference='port') has been forced to '" + portNumberValue + "' for account '" + account.key + "'");
	}

	var username = ( (accountServer.realUsername != null) && (accountServer.realUsername != "") ) ?
						accountServer.realUsername :
						( (accountServer.username != null) && (accountServer.username != "") ) ?
							accountServer.username :
							"<undefined>";
	var hostname = ( (accountServer.realHostName != null) && (accountServer.realHostName != "") ) ?
						accountServer.realHostName :
						( (accountServer.hostName != null) && (accountServer.hostName != "") ) ?
							accountServer.hostName :
							"<undefined>";

	try {
		var accountInfo = "'" + account.key + "' (" + accountServer.prettyName + ": " + username + "/" + hostname + "/" + accountServer.type + "/" + accountServer.port + ")";

		switch ( accountServer.type ) { // according to account server type
			case "local":
			case "movemail":
			case "pop3":
			case "imap":
			case "rss":
				this.localConsole.logInformation("getMessages", "Getting new messages for account " + accountInfo + "...");
				accountServer.getNewMessages(accountServer.rootMsgFolder, msgWindow, null); // gets new messages
				break;
			case "nntp":
				this.localConsole.logInformation("getMessages", "Getting new messages for account " + accountInfo + "...");
				accountServer.performExpand(msgWindow); // gets new messages
				break;
			case "none": // Local Folders
				break;
			default:
				this.localConsole.logInformation("getMessages", "Account type '" + accountServer.type + "' of account " + accountInfo + " is not handled");
		}
	}
	catch ( err ) {
		this.localConsole.logError("getMessages", "Error while retrieving new messages for " + accountServer.serverURI + " (" + err + ")");
	}

	// back to default headers fetching settings
	if ( (headersFetchingValue != null) && (headersFetchingValue != 0) && !profileSettings.hdrsDflt[accountKey] ) { // if headers fetching value is not set to 'keep default value' and 'set as default' is disabled
		serverPrefs.setBoolPref("headers_only", accountServerHeadersOnly); // restores default 'fetch headers only' value

		this.localConsole.logInformation("getMessages", "Headers fetching value (preference='headers_only') has been reset to default value ('" + accountServerHeadersOnly + "') for account '" + account.key + "'");
	}

	// back to default server settings
	if ( (serverValue != null) && serverValue && (serverNameValue != null) && (serverNameValue != "") && !profileSettings.srvDflt[accountKey] ) { // if server value is enabled and server name is not empty and 'set as default' is disabled
		if ( accountServerRealHostName != "" ) {
			serverPrefs.setCharPref("realhostname", accountServerRealHostName); // restores default 'server name' value

			this.localConsole.logInformation("getMessages", "Server value (preference='realhostname') has been reset to default value ('" + accountServerRealHostName + "') for account '" + account.key + "'");
		}
		else if ( accountServerHostName != "" ) {
			serverPrefs.setCharPref("hostname", accountServerHostName); // restores default 'server name' value

			this.localConsole.logInformation("getMessages", "Server value (preference='hostname') has been reset to default value ('" + accountServerHostName + "') for account '" + account.key + "'");
		}
	}

	// back to default port settings
	if ( (portValue != null) && portValue && (portNumberValue != null) && (portNumberValue != 0) && !profileSettings.prtDflt[accountKey] ) { // if port value is enabled and port is not 0 and 'set as default' is disabled
		if ( accountServerPortExist ) {
			serverPrefs.setIntPref("port", accountServerPort); // restores default 'port' value
		}
		else {
			serverPrefs.clearUserPref("port");
		}

		var accountServerPortMsg = ( accountServerPort > 0 ) ? "('" + accountServerPort + "') " : "";
		this.localConsole.logInformation("getMessages", "Port number value (preference='port') has been reset to default value " + accountServerPortMsg + "for account '" + account.key + "'");
	}
};

/**
 * Detects profile using detection rules
 *
 * @param getMessagesFlag (boolean) true/false if messages should be/not be retrieved after profile detection
 */
GetSelectedMails.launchProfileDetection = function (getMessagesFlag)
{
	var dnsService = Components.classes["@mozilla.org/network/dns-service;1"]
						.createInstance(Components.interfaces.nsIDNSService);

	var dnsResolveThread = Components.classes["@mozilla.org/thread-manager;1"]
							.getService(Components.interfaces.nsIThreadManager).currentThread;

	this.localConsole.logInformation("launchSMTPServerDetection", "Retrieving IP addresses for local host '" + dnsService.myHostName + "'...");

	dnsService.asyncResolve(dnsService.myHostName, Components.interfaces.nsIDNSService.RESOLVE_CANONICAL_NAME | Components.interfaces.nsIDNSService.RESOLVE_DISABLE_IPV6, new GetSelectedMails.DNSResolveObserver(dnsService.myHostName, getMessagesFlag), dnsResolveThread);

	// process will be pursued in the DNSResolveObserver.onLookupComplete handler (finalizeProfileDetection()) once DNS request will be completed
};

/**
 * Detects profile using detection rules
 *
 * @param message (string) message to log
 * @param getMessagesFlag (boolean) true/false if messages should be/not be retrieved after profile detection
 */
GetSelectedMails.finalizeProfileDetection = function (dnsRecords, getMessagesFlag)
{
	var reIPAddress = new RegExp("^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$", "i");

	var ipAddress = "";

	var startIP = "";
	var endIP = "";
	var profileId = "";

	var ipParts = null;
	var startIPParts = null;
	var endIPParts = null;

	var okFlag = true;
	var foundFlag = false;
	var foundIdx = -1;

	// extraction of detection rules list
	var detectionRulesArray = null;

	if ( (this.detectionRules == null) || (this.detectionRules == "") ) { // if detection rules list is empty
		detectionRulesArray = new Array();
	}
	else {
		detectionRulesArray = GetSelectedMails.JSONparse(this.detectionRules, GetSelectedMails.preProcessLoadSettings, new Array("startIP", "endIP", "profileId"));
	}

	while ( dnsRecords.hasMore() ) {
		ipAddress = dnsRecords.getNextAddrAsString();

		if ( !foundFlag && (foundIdx == -1) ) {
			this.localConsole.logInformation("finalizeProfileDetection", "Checking " + ipAddress + "...");

			ipParts = ipAddress.match(reIPAddress);

			if ( ipParts != null ) {
				foundFlag = false;

				for ( var idx = 0 ; idx < detectionRulesArray.length && !foundFlag ; idx++ ) { // for each detection rule
					startIP = detectionRulesArray[idx].sIP;
					endIP = detectionRulesArray[idx].eIP;
					profileId = detectionRulesArray[idx].pId;

					this.localConsole.logInformation("finalizeProfileDetection", "Checking rule [" + startIP + ";" + endIP + "] => " + profileId);

					startIPParts = startIP.match(reIPAddress);
					endIPParts = endIP.match(reIPAddress);

					if ( (startIPParts != null) && (endIPParts != null) ) {
						okFlag = true;

						// check of IP
						for ( var jdx = 1 ; (jdx <= 4) && okFlag ; jdx++ ) {
							if ( (parseInt(ipParts[jdx]) < parseInt(startIPParts[jdx])) || (parseInt(ipParts[jdx]) > parseInt(endIPParts[jdx])) ) {
								okFlag = false;
							}
						}

						if ( okFlag ) {
							this.localConsole.logInformation("finalizeProfileDetection", "IP address " + ipAddress + " validates rule [" + startIP + ";" + endIP + "] => " + profileId);

							if ( detectionRulesArray[idx].on ) {
								foundFlag = true;
							}
							else {
								this.localConsole.logInformation("finalizeProfileDetection", "Rule [" + startIP + ";" + endIP + "] => " + profileId + " is disabled: skipped");
							}
						}
					}
					else {
						this.localConsole.logError("finalizeProfileDetection", "Rule [" + startIP + ";" + endIP + "] => " + profileId + " is invalid");
					}
				}

				if ( foundFlag ) {
					foundIdx = -1;

					if ( this.profilesSettingsList[this.defaultProfile].id != profileId ) { // if detected profile is different from current default profile
						for ( var idx = 0 ; (idx < this.profilesSettingsList.length) && (foundIdx == -1) ; idx++ ) {
							if ( this.profilesSettingsList[idx].id == profileId ) {
								foundIdx = idx;
							}
						}

						if ( foundIdx != -1 ) {
							this.defaultProfile = foundIdx;

							this.localConsole.logInformation("finalizeProfileDetection", "Default profile has been changed to #" + foundIdx + " / " + this.profilesSettingsList[foundIdx].id + " / '" + this.profilesSettingsList[foundIdx].name + "'");

							this.prefs.setIntPref("defaultProfile", this.defaultProfile);
						}
						else {
							this.localConsole.logError("finalizeProfileDetection", "Detected profile has not been found (" + profileId + ")");
						}
					}
					else { // if detected profile is not different from current default profile
						this.localConsole.logInformation("finalizeProfileDetection", "Default profile has not been changed (#" + this.defaultProfile + " / " + profileId + " / '" + this.profilesSettingsList[this.defaultProfile].name + "')");
					}
				}
				else {
					this.localConsole.logInformation("finalizeProfileDetection", "No rule could have been validated");
				}
			}
			else {
				this.localConsole.logWarning("finalizeProfileDetection", ipAddress + " is not a valid IPv4 address");				
			}
		}
		else {
			this.localConsole.logWarning("finalizeProfileDetection", "Detection profile has already been validated: check of " + ipAddress + " skipped");
		}
	}

	if ( foundIdx == -1 ) {
		this.selectDefaultProfile(this.defaultProfile);
	}

	if ( this.currentDefaultProfileFlag ) {
		this.currentProfile = this.defaultProfile;
	}

	if ( getMessagesFlag ) {
		this.finalizeGetMessages();
	}
};

/**
 * Handles profile detection timer
 *
 * @param autoDetectionAtIntervals (boolean) true/false if profile detection timer has to be started/stopped
 */
GetSelectedMails.launchDetectProfileTimer = function (autoDetectionAtIntervals)
{
	if ( autoDetectionAtIntervals ) {
		if ( this.detectTimer == null ) {
			this.detectTimer = Components.classes["@mozilla.org/timer;1"]
								.createInstance(Components.interfaces.nsITimer);
		}
		this.detectTimer.cancel();

		this.detectTimer.initWithCallback(new GetSelectedMails.TimerDetectProfile(this.detectTimer), this.intervals * 60000, Components.interfaces.nsITimer.TYPE_REPEATING_SLACK); // starts timer

		this.localConsole.logInformation("launchDetectProfileTimer", "Profile detection timer has been started");
	}
	else {
		if ( this.detectTimer != null ) {
			this.detectTimer.cancel();

			this.localConsole.logInformation("launchDetectProfileTimer", "Profile detection timer has been stopped");
		}
	}
};

/**
 * Modifies profile detection timer interval
 */
GetSelectedMails.modifyDetectProfileTimerInterval = function ()
{
	this.launchDetectProfileTimer(false);
	this.launchDetectProfileTimer(true);
};

/**
 * Detects profile automatically using detection rules
 */
GetSelectedMails.detectProfileAuto = function ()
{
	if ( this.autoDetectionAtStart ) {
		this.launchProfileDetection(false);
	}
	else {
		this.localConsole.logInformation("detectProfileAuto", "Profile auto detection is disabled");

		this.selectDefaultProfile(this.defaultProfile);
	}
};

/**
 * Finishes add-on execution
 */
GetSelectedMails.shutdown = function ()
{
	this.prefs.removeObserver("", this);
};

/**
 * Observes modifications of settings
 *
 * @param prefBranch (nsIPrefBranch) observed preferences change
 * @param topic (string) fired event
 * @param data (string) name of changed preference
 */
GetSelectedMails.observe = function (prefBranch, topic, data)
{
	if ( topic == "nsPref:changed" ) {
		switch ( data ) {
			case "profilesSettings": this.profilesSettings = this.prefs.getCharPref("profilesSettings"); break;
			case "defaultProfile": this.selectDefaultProfile(this.prefs.getIntPref("defaultProfile")); break;
			case "mainButtonEnabled":
				this.mainButtonEnabled = this.prefs.getBoolPref("mainButtonEnabled");
				this.displayCustomButton("getSelectedMailsButton", this.mainButtonEnabled, "button-newmsg");

				this.profilesListChanged = true;
				this.updateProfilesList(false); // updates profiles list
				this.selectDefaultProfile(this.defaultProfile);
				break;
			case "autoDetectionAtStart":
				this.autoDetectionAtStart = this.prefs.getBoolPref("autoDetectionAtStart");
				this.detectProfileAuto();
				break;
			case "autoDetectionAtGetMails": this.autoDetectionAtGetMails = this.prefs.getBoolPref("autoDetectionAtGetMails"); break;
			case "autoDetectionAtIntervals":
				this.autoDetectionAtIntervals = this.prefs.getBoolPref("autoDetectionAtIntervals");
				this.launchDetectProfileTimer(this.autoDetectionAtIntervals);
				break;
			case "intervals":
				this.intervals = this.prefs.getIntPref("intervals");
				this.modifyDetectProfileTimerInterval();
				break;
			case "detectionRules":
				this.detectionRules = this.prefs.getCharPref("detectionRules");
				this.detectProfileAuto();
				break;
			case "delayEnabled": this.delayEnabled = this.prefs.getBoolPref("delayEnabled"); break;
			case "delay": this.delay = this.prefs.getIntPref("delay"); break;
			case "delayOnlyDeferredAccounts": this.delayOnlyDeferredAccounts = this.prefs.getBoolPref("delayOnlyDeferredAccounts"); break;
			case "nativeGetMailButtonOptionsEnabled": this.enableGetMailButtonOptions(this.prefs.getBoolPref("nativeGetMailButtonOptionsEnabled")); break;
			case "debugMode": this.debugMode = this.prefs.getBoolPref("debugMode"); break;
			case "debugModeToFile": this.debugModeToFile = this.prefs.getBoolPref("debugModeToFile"); break;
			case "debugFileName": this.debugFileName = unescape(this.prefs.getCharPref("debugFileName")); break;

			case "profilesListChanged":
				this.profilesListChanged = this.prefs.getBoolPref("profilesListChanged");
				this.updateProfilesList(false); // updates profiles list
				this.selectDefaultProfile(this.defaultProfile);
				break;
		}
	}
};


/**
 * (class) Manages behaviour when a 'get new messages' timer has expired
 *
 * @param timer (nsITimer) timer object which has expired
 * @param profileSettings (ProfileSettings object) settings of current profile as a ProfileSettings object
 * @param account (nsIMsgAccount) account object to retrieve new messages for
 */
GetSelectedMails.TimerGetMessages = function (timer, profileSettings, account)
{
	// handles behaviour when timer has expired (overriding of inherited method from nsITimer interface)
	this.notify = function (timer) {
		GetSelectedMails.localConsole.logInformation("TimerGetMessages.notify", "Timer for account '" + account.key + "' has expired");

		GetSelectedMails.getMessages(profileSettings, account); // starts retrieving new messages for given account of given profile
	}
};


/**
 * (class) Manages behaviour when a 'detect profile' timer has expired
 *
 * @param timer (nsITimer) timer object which has expired
 */
GetSelectedMails.TimerDetectProfile = function (timer, profileSettings, account)
{
	// handles behaviour when timer has expired (overriding of inherited method from nsITimer interface)
	this.notify = function (timer) {
		GetSelectedMails.localConsole.logInformation("TimerDetectProfile.notify", "Profile detection timer has expired");

		GetSelectedMails.launchProfileDetection(false); // detects profile
	}
};



/**
 * (class) Manages behaviour when a DNS request has been completed
 *
 * @param hostName (string) host name
 * @param getMessagesFlag (boolean) true/false if messages should be/not be retrieved after profile detection
 */
GetSelectedMails.DNSResolveObserver = function (hostName, getMessagesFlag)
{
	this.onLookupComplete = function (request, records, status) {
		if ( Components.isSuccessCode(status) ) {
			GetSelectedMails.finalizeProfileDetection(records, getMessagesFlag); // finalizes IP detection
		}
		else {
			GetSelectedMails.localConsole.logError("DNSResolveObserver.onLookupComplete", "Error while retrieving IP addresses for local host '" + hostName + "'");
		}
	}
};


window.addEventListener("load", function() { GetSelectedMails.initializeOverlayStartup(); }, false);
window.addEventListener("unload", function() { GetSelectedMails.shutdown(); }, false);
